//
//  AdColonyNetwork.m
//  SampleApp
//
//  Created by Pokkt on 10/16/15.
//  Copyright © 2015 Pokkt. All rights reserved.
//

#import "AdColonyNetwork.h"
#import "PokktSDK/PokktSDK.h"
#import "PokktSDK/PokktNetworkDelegate.h"

@implementation AdColonyNetwork

#define versionNumber @"3.2.0"

static bool isInitialized;
static NSString *AppId;
static NSArray *allZoneIds;

#pragma mark PokktAdNetwork Method

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    if (isInitialized)
    {
        [PokktDebugger printLog:@"[AdColony SDK]Already Initialised Adcolony Network"];
        return;
    }
     isInitialized = YES;
    [PokktDebugger printLog:@"[AdColony SDK] Initialising Adcolony Network"];
    adConfigDict = [[NSMutableDictionary alloc] initWithCapacity:0];
    
    AppId = [[customData.customData objectForKey:@"appId"] copy];
    allZoneIds = [[[customData.customData objectForKey:@"allZoneIds"] componentsSeparatedByString:@","] copy];
    self.network = customData;
    
    self.adColonyIntMap = [[NSMutableDictionary alloc] initWithCapacity:0];
    
    if (AppId != nil && AppId.length > 0  && allZoneIds.count)
    {
        [AdColony configureWithAppID:AppId zoneIDs:allZoneIds options:nil completion:^(NSArray<AdColonyZone *> * zones)
        {
            AdColonyZone *zone = [zones firstObject];
            __block NSString *zoneID = zone.identifier;
            zone.reward = ^(BOOL success, NSString *name, int amount)
            {
                if (success)
                {
                    [self adGratified:zoneID withAmout:amount];
                }
            };
         }];
    }
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    NSString *screenName = [self getScreenName:adConfig];
    
    if (screenName == nil && screenName.length == 0)
    {
        [PokktDebugger printLog:@"[AdColony SDK]Caching failed.... ScreenName cannot be nil to cacheAd"];
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:@"[AdColony SDK] Invalid network"];
        return;
    }
    
    [self cacheAdWithScreenName:screenName PokktAdConfig:adConfig];
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    
    NSString *screenName = [self getScreenName:adConfig];
    
    if (screenName == nil && screenName.length == 0)
    {
        [PokktDebugger printLog:@"[AdColony SDK] ScreenName cannot be nil for showAd"];
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
        return;
    }
    
    if ([allZoneIds containsObject:screenName])
    {
        [adConfigDict setObject:adConfig forKey:screenName];
        AdColonyInterstitial *ad = (AdColonyInterstitial *)[self.adColonyIntMap objectForKey:screenName];
        if (!ad.expired)
        {
            [PokktDebugger printLog:@"[AdColony SDK]AdColony Ad start Play"];
            [ad showWithPresentingViewController:viewController];
        }
        else
        {
            [PokktDebugger printLog:@"[AdColony SDK]AdColony Ad failed to show"];
            [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
        }
    }
    else
    {
        [PokktDebugger printLog:@"[AdColony SDK]AdColony Ad failed to show"];
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
    }
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.network;
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;
    
    NSString *screenName = [self getScreenName:adConfig];
    
    if ([self.adColonyIntMap objectForKey:screenName] == nil)
    {
        [PokktDebugger printLog:@"[AdColony SDK]Ad available status is false.... due to nil screenname"];
        [PokktNetworkDelegate onAdAvailabilityStatus:self.network adConfig:adConfig isAdAvailable:NO];
        return;
    }

    AdColonyInterstitial *ad = (AdColonyInterstitial *)[self.adColonyIntMap objectForKey:screenName];
    if (!ad.expired)
    {
        flag = YES;
        [PokktDebugger printLog:@"[AdColony SDK]Ad available status is true"];

    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.network adConfig:adConfig isAdAvailable:flag];

}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    [PokktDebugger printLog:@"[AdColony SDK] Caching time out"];
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
        return true;
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        return true;
    }
    
    return false;
}

#pragma mark - AdColony V4VC

-(void)adOpen:(NSString *)zoneID
{
    [PokktDebugger printLog:@"[AdColony SDK] AdColony Ad Displayed"];
    PokktAdConfig *adconfig = (PokktAdConfig *)[adConfigDict objectForKey:zoneID];
    [PokktNetworkDelegate onAdDisplayed:self.network adConfig:adconfig];
}

-(void)adClose:(NSString *)zoneID
{
    PokktAdConfig *adconfig = (PokktAdConfig *)[adConfigDict objectForKey:zoneID];
    [PokktDebugger printLog:@"[AdColony SDK] AdColony Ad completed"];
    [PokktNetworkDelegate onAdCompleted:self.network adConfig:adconfig];
    [PokktNetworkDelegate onAdClosed:self.network adConfig:adconfig];
    
    [adConfigDict removeObjectForKey:zoneID];
    [self.adColonyIntMap removeObjectForKey:zoneID];
}

-(void)adGratified:(NSString *)zoneID withAmout:(int)amount
{
    [PokktDebugger printLog:@"[AdColony SDK] AdColony Ad completed and gratified"];
    PokktAdConfig *adConfig = [adConfigDict objectForKey:zoneID];
    if (adConfig != nil && adConfig.isRewarded)
    {
        [PokktNetworkDelegate onAdGratified:self.network adConfig:[adConfigDict objectForKey:zoneID] rewardPoint:amount];
    }
    
}


#pragma mark Cache Mapping Method

-(void)cacheAdWithScreenName:(NSString *)screen PokktAdConfig:(PokktAdConfig *)adConfig
{
  if ([allZoneIds containsObject:screen])
  {
    [adConfigDict setObject:adConfig forKey:screen];
    [AdColony requestInterstitialInZone:screen options:nil
     
                                success:^(AdColonyInterstitial* ad) {
                                    
                                    ad.open = ^{
                                        [self adOpen:screen];
                                    };
                                    
                                    ad.close = ^{
                                        [self adClose:screen];
                                    };
                                    
                                    ad.expire = ^{
                                        
                                     };
                                    
                                    [PokktDebugger printLog:@"[AdColony SDK] AdColony Ad is available to watch"];
                                    [PokktNetworkDelegate didFinishedAdDownload:self.network adConfig:adConfig rewardValue:0.0 downLoadTime:@""];
                                    
                                    [self.adColonyIntMap setObject:ad forKey:screen];
                                }
     
                                failure:^(AdColonyAdRequestError* error)
                                {
                                    NSString *errorMsg = [NSString stringWithFormat:@"[AdColony SDK] Request failed with error: %@ and suggestion: %@",[error localizedDescription],[error localizedRecoverySuggestion]];
                                    [PokktDebugger printLog:errorMsg];
                                    [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:errorMsg];
                                }
     ];
    }
    else
    {
        [PokktDebugger printLog:@"[AdColony SDK]Caching failed.... Screenname is not valid"];
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:@"[AdColony SDK] Invalid network"];
    }

}

#pragma mark Get ScreenName from Custom Data

-(NSString *)getScreenName:(PokktAdConfig *)adConfig
{
    NSString *screenName = @"";
    
    if ([self.network.customData objectForKey:@"screens_mapping_data"] != nil && [self.network.customData objectForKey:@"screens_mapping_data"])
    {
        NSDictionary *screenDict = [self.network.customData objectForKey:@"screens_mapping_data"];
        
        if ([screenDict objectForKey:adConfig.screenName] != nil && [screenDict objectForKey:adConfig.screenName])
        {
            NSDictionary *networkDict = [screenDict objectForKey:adConfig.screenName];
            
            if ([networkDict objectForKey:@"network_screen"] != nil && [networkDict objectForKey:@"network_screen"])
            {
                screenName = [networkDict objectForKey:@"network_screen"];
            }
        }
    }
    else
    {
        screenName = adConfig.screenName;
    }
    
    return screenName;
}


@end
