//
//  AerServNetwork.m
//  PokktAdsDemo
//
//  Created by Pokkt on 28/02/17.
//  Copyright © 2017 Pokkt. All rights reserved.
//

#import "AerServNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"



#define versionNumber @"2.41.1"


@implementation AerServNetwork


- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.networkModel = customData;
    
    NSString *appId = [customData.customData objectForKey:@"appId"];//@"1000834"
    if (appId != nil && appId.length > 0)
    {
        [AerServSDK showLogs:YES];
        [AerServSDK initializeWithAppID:appId];
    }

    
    self.interstitialAd = [[AerServInterstitialAds alloc] init];
    [self.interstitialAd initNetworkWithNetworkModel:customData];
    
    self.bannerAd = [[AerServBanner alloc] init];
    self.bannerAd.networkModel = customData;
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    [self.interstitialAd cacheAd:adConfig];
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    [self.interstitialAd showAd:adConfig viewController:viewController];
}

-(void) fetchAd:(PokktAdConfig *)adConfig withAdView:(UIView *)bannerView withRootViewController:(UIViewController *)rootViewController withDelegate:(id<PokktBannerDelegate>)bannerDelegate onSuccess:(void (^)(id))successCallback onFailure:(void (^)(id))failureHandler
{
    [self.bannerAd fetchAd:adConfig withAdView:bannerView withRootViewController:rootViewController withDelegate:bannerDelegate onSuccess:successCallback onFailure:failureHandler];
}


-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    [self.interstitialAd checkAdAvailability:adConfig];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    [self.interstitialAd setCacheTimedOut:adConfig];
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == BANNER)
    {
        flag = [self.bannerAd checkAdFormatSupport:adConfig];
    }
   else if (adConfig.adFormat == INTERSTITIAL)
    {
        flag = [self.interstitialAd checkAdFormatSupport:adConfig];
    }
    
    return flag;
}


@end

#pragma mark Interstitial Network

@implementation AerServInterstitialAds

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    NSLog(@"AerServ is initialised");
    
    self.placementDictionary = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.adConfigDictionary = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.timeoutDictionary = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.networkModel = customData;
    
    
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    ASInterstitialViewController *placement = (ASInterstitialViewController *)[self getPlacementwithAdConfig:adConfig];
    
    if (placement == nil)
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"AerServ placement content is not available"];
        return;
    }
    
    [self.placementDictionary setObject:placement forKey:adConfig.getKey];
    [self.adConfigDictionary setObject:adConfig forKey:placement.placementID];
    
    if (placement.isReady)
    {
        NSLog(@"AerServ cached previously");
        
        [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
    }
    else
    {
        NSLog(@"AerServ caching started");
        
        [self.timeoutDictionary setObject:[NSNumber numberWithBool:NO] forKey:adConfig.getKey];
        [placement loadAd];
        
    }

}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    ASInterstitialViewController *placement = [self getPlacementwithAdConfig:adConfig];
    
    if (placement != nil && placement.isReady)
    {
        [placement showFromViewController:viewController];
    }
    else
    {
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;
    
    ASInterstitialViewController *placement = [self getPlacementwithAdConfig:adConfig];
    
    if (placement != nil && placement.isReady)
    {
        flag = YES;
    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"AerServ Caching time out");
    [self.timeoutDictionary setObject:[NSNumber numberWithBool:YES] forKey:adConfig.getKey];
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == INTERSTITIAL)
    {
        flag = true;
    }
    
    return flag;
}

-(ASInterstitialViewController *)getPlacementwithAdConfig:(PokktAdConfig *)adConfig
{
    ASInterstitialViewController *placement;
    
    if ([self.placementDictionary objectForKey:adConfig.getKey])
    {
        placement = [self.placementDictionary objectForKey:adConfig.getKey];
    }
    else
    {
        NSDictionary *screenMapdictionary = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
        
        if (screenMapdictionary != nil && [screenMapdictionary objectForKey:adConfig.screenName])
        {
            NSDictionary * networkScreenDetails = [screenMapdictionary objectForKey:adConfig.screenName];
            
            NSString * placementName = [networkScreenDetails objectForKey:@"network_screen"];
            
            if (placementName != nil && placementName.length !=0)
            {
                placement = [ASInterstitialViewController viewControllerForPlacementID:placementName withDelegate:self];
            }
        }
    }
    
    return placement; //1000836
    
}

#pragma mark Delegates

- (void)interstitialViewControllerAdLoadedSuccessfully:(ASInterstitialViewController*)viewController
{
    NSLog(@"AerServ Interstitial did load.");
    
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:viewController.placementID];
    
    if (adConfig != nil)
    {
        if ([[self.timeoutDictionary objectForKey:adConfig.getKey] boolValue])
        {
            NSLog(@"AerServ caching failed after timeout");
        }
        else
        {
            NSLog(@"AerServ download completed");
            
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
        }
    }
    else
    {
        NSLog(@"Unknown placement");
    }

}

- (void)interstitialViewControllerDidPreloadAd:(ASInterstitialViewController *)viewController {
    
}

- (void)interstitialViewControllerAdFailedToLoad:(ASInterstitialViewController *)viewController withError:(NSError *)error {
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:viewController.placementID];
    
    if (adConfig)
    {
        if ([[self.timeoutDictionary objectForKey:adConfig.getKey] boolValue])
        {
            NSLog(@"AerServ caching failed after timeout");
        }
        else
        {
            NSLog(@"AerServ download failed");
            
            [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:error.description];
        }
    }

}

- (void)interstitialViewControllerWillAppear:(ASInterstitialViewController *)viewController {
    NSLog(@"Interstitial will display.");
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:viewController.placementID];
    
    if (adConfig)
    {
        [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:adConfig];
    }
    else
    {
        NSLog(@"Unknown placement");
    }
}

- (void)interstitialViewControllerDidAppear:(ASInterstitialViewController *)viewController {
    
}

- (void)interstitialViewControllerWillDisappear:(ASInterstitialViewController *)viewController {
    NSLog(@"Interstitial will dismiss.");
    
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:viewController.placementID];
    
    if (adConfig)
    {
        [self.placementDictionary removeObjectForKey:adConfig.getKey];
        [self.adConfigDictionary removeObjectForKey:viewController.placementID];
        [self.timeoutDictionary removeObjectForKey:adConfig.getKey];
        [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:adConfig];
    }
    else
    {
        NSLog(@"Unknown placement");
    }
}

- (void)interstitialViewControllerDidDisappear:(ASInterstitialViewController *)viewController
{
    
}

- (void)interstitialViewControllerAdWasTouched:(ASInterstitialViewController *)viewController
{
    
}

- (void)interstitialViewControllerDidVirtualCurrencyLoad:(ASInterstitialViewController *)viewController vcData:(NSDictionary *)vcData
{
    NSLog(@"Currency INFO %@",vcData);
    if (vcData != nil && [[vcData objectForKey:@"rewardAmount"] floatValue] > 0)
    {
        PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:viewController.placementID];
        [PokktNetworkDelegate onAdGratified:self.networkModel adConfig:adConfig rewardPoint:[[vcData objectForKey:@"rewardAmount"] floatValue]];

    }
}

- (void)interstitialViewControllerDidVirtualCurrencyReward:(ASInterstitialViewController *)viewController vcData:(NSDictionary *)vcData
{
    NSLog(@"Currency INFO %@",vcData);
}

@end;

@implementation AerServBanner

-(void) fetchAd:(PokktAdConfig *)adConfig withAdView:(UIView *)bannerView withRootViewController:(UIViewController *)rootViewController withDelegate:(id<PokktBannerDelegate>)bannerDelegate onSuccess:(void (^)(id))successCallback onFailure:(void (^)(id))failureHandler
{
    NSLog(@"Aerserv banner fetch request");
    self.bannerDelegate = bannerDelegate;
    self.rootViewController = rootViewController;
    NSString *placementId = [self getPlacementIdwithAdConfig:adConfig]; //1000834
    if (placementId != nil && placementId.length != 0)
    {
        ASAdView *asAdView = [ASAdView viewWithPlacementID:placementId andAdSize:ASBannerSize];
        asAdView.delegate = self;
        [asAdView loadAd];
        [bannerView addSubview:asAdView];
    }
    else
    {
        [self.bannerDelegate bannerLoadFailed:adConfig.screenName errorMessage:@"[AerServ SDK] Invalid adConfig"];
    }
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == BANNER)
    {
        return true;
    }
    return false;
}

-(NSString *)getPlacementIdwithAdConfig:(PokktAdConfig *)adConfig
{
    NSString *placementID;
    
    NSDictionary *screenMapdictionary = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
    
    if (screenMapdictionary != nil && [screenMapdictionary objectForKey:adConfig.screenName])
    {
        NSDictionary * networkScreenDetails = [screenMapdictionary objectForKey:adConfig.screenName];
        placementID = [networkScreenDetails objectForKey:@"network_screen"];
    }
    
    return placementID;
}

- (UIViewController*)viewControllerForPresentingModalView
{
    return self.rootViewController;
}

- (void)adViewDidLoadAd:(ASAdView*)adView
{
    NSLog(@"AerServ Banner Loaded");
    [self.bannerDelegate bannerLoaded:@"AerServ Banner Loaded Successfully"];
}

- (void)adViewDidFailToLoadAd:(ASAdView*)adView withError:(NSError*)error
{
    NSLog(@"AerServ Banner Load failed");
    [self.bannerDelegate bannerLoadFailed:@"AerServ Banner Loaded failed" errorMessage:error.description];
}





@end
