//
//  AppLovin.m
//  SampleApp
//
//  Created by Pokkt on 10/8/15.
//  Copyright © 2015 Pokkt. All rights reserved.
//

#import "AppLovin.h"
#import <AppLovinSDK/ALSdk.h>
#import <AppLovinSDK/ALIncentivizedInterstitialAd.h>

#import "PokktSDK/PokktNetworkDelegate.h"
@implementation AppLovin

#define versionNumber @"4.3.1"

static BOOL isInitialized;

ALInterstitialAd *interstitial;

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    if (isInitialized)
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Already Initialised AppLovin Network"];
        return;
    }
    
    [PokktDebugger printLog:@"[AppLovin SDK]Initialising AppLovin Network"];
    isInitialized = YES;
    [ALSdk initializeSdk];
    self.networkModel = customData;
    self.appLovinRewardedAd = [[RewardedAD alloc] init];
    [self.appLovinRewardedAd initNetworkWithNetworkModel:customData];
    self.appLovinInterstitialAd = [[InterstitialAd alloc] init];
    [self.appLovinInterstitialAd initNetworkWithNetworkModel:customData];
}


- (void) cacheAd: (PokktAdConfig *)adConfig
{
    NSString *str = nil;
    
    if (adConfig.adFormat == VIDEO)
    {
        [self.appLovinRewardedAd cacheAd:adConfig];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.appLovinInterstitialAd cacheAd:adConfig];
        
    }
    NSLog(@"%@",str);
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    
     if (adConfig.adFormat == VIDEO)
    {
        [self.appLovinRewardedAd showAd:adConfig viewController:viewController];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.appLovinInterstitialAd showAd:adConfig viewController:viewController];
    }
    
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO) {
        [self.appLovinRewardedAd checkAdAvailability:adConfig];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.appLovinInterstitialAd checkAdAvailability:adConfig];
    }
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
        [self.appLovinRewardedAd setCacheTimedOut:adConfig];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.appLovinInterstitialAd setCacheTimedOut:adConfig];
    }
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;
    
    if (adConfig.adFormat == VIDEO)
    {
        flag = [self.appLovinRewardedAd checkAdFormatSupport:adConfig];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        flag = [self.appLovinInterstitialAd checkAdFormatSupport:adConfig];
    }
    
    return flag;
}




@end

@interface RewardedAD()
@property(atomic)BOOL isCachingAvailable;
@property(atomic,strong) NSString *screenName;
@end

static NSString *rewardAmount;

@implementation RewardedAD

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.network = customData;
    self.isTimedout = NO;
    
    self.isCachingAvailable = YES;
    self.screenName = nil;
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    
    if (self.screenName != nil && ![self.screenName isEqualToString:adConfig.screenName])
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Video Caching failed.... Invalid network"];
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig
                                     errorMessage:@"Caching in AppLovin Failed"];
        return;
    }
    
    if (self.isCachingAvailable)
    {
        self.isCachingAvailable = NO;
        self.screenName = adConfig.screenName;
        self.adConfig = adConfig;
        if ([ALIncentivizedInterstitialAd isReadyForDisplay])
        {
            [PokktDebugger printLog:@"[AppLovin SDK]Video were already saved in cached"];
            [PokktNetworkDelegate didFinishedAdDownload:self.network adConfig:adConfig rewardValue:0 downLoadTime:@""];
        }
        else
        {
            [PokktDebugger printLog:@"[AppLovin SDK]Video caching start with timeout timer"];
            [ALIncentivizedInterstitialAd shared].adDisplayDelegate = self;
            [ALIncentivizedInterstitialAd shared].adVideoPlaybackDelegate = self;
            [[ALIncentivizedInterstitialAd shared] preloadAndNotify: self];
            self.isTimedout = NO;
        }
    }
    else
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Video caching failed "];
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:@"[APP-LOVIN]Video caching failed"];
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    self.adConfig = adConfig;
    if (self.screenName == nil || ![self.screenName isEqualToString:adConfig.screenName])
    {
         [PokktDebugger printLog:@"[AppLovin SDK]ScreenName cannot be nil to show Video Ad"];
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
        return;
    }
    
    if ([ALIncentivizedInterstitialAd isReadyForDisplay])
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Video start"];
        [ALIncentivizedInterstitialAd showAndNotify:self];
    }
    else
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Video Play failed to show"];
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
    }
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    if (self.screenName == nil || ![self.screenName isEqualToString:adConfig.screenName])
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Video available status is false.... due to nil screenname"];
        [PokktNetworkDelegate onAdAvailabilityStatus:self.network adConfig:adConfig isAdAvailable:NO];
        return;
    }
    
    BOOL flag = NO;
    if (adConfig.isRewarded) {
        if ([ALIncentivizedInterstitialAd isReadyForDisplay]) {
            
            flag = YES;
        }
    }
    [PokktNetworkDelegate onAdAvailabilityStatus:self.network adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    [PokktDebugger printLog:@"[AppLovin SDK]Video Caching time out"];
    self.isTimedout = YES;
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.network;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == 0)
    {
        flag = true;
    }
    
    return flag;
}



#pragma mark - Rewarded Ad Load Delegate

-(void) ad: (alnonnull ALAd *) ad wasDisplayedIn: (alnonnull UIView *) view
{
    [PokktDebugger printLog:@"[AppLovin SDK]Video displayed"];
    [PokktNetworkDelegate onAdDisplayed:self.network adConfig:self.adConfig];
}

-(void) ad: (alnonnull ALAd *) ad wasHiddenIn: (alnonnull UIView *) view
{
    [PokktDebugger printLog:@"[AppLovin SDK]Video closed"];
    if (self.adConfig.isRewarded) {
        self.isCachingAvailable = YES;
    }
    self.screenName = nil;
    [PokktNetworkDelegate onAdClosed:self.network adConfig:self.adConfig];
}

- (void)adService:(ALAdService *)adService didLoadAd:(ALAd *)ad
{
    if (self.isTimedout)
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Video Caching failed with timeOut"];
    }
    else
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Video downloaded"];
        [PokktNetworkDelegate didFinishedAdDownload:self.network adConfig:self.adConfig rewardValue:0 downLoadTime:@""];
    }
}

-(void) ad: (alnonnull ALAd *) ad wasClickedIn: (alnonnull UIView *) view
{
    [PokktDebugger printLog:@"[AppLovin SDK]Clicked on video"];
}

- (void)adService:(ALAdService *)adService didFailToLoadAdWithError:(int)code
{
    NSString *errorMsg = [NSString stringWithFormat: @"[APP-LOVIN]Video failed to load with error code %d", code];
    if (self.isTimedout)
    {
        errorMsg = [NSString stringWithFormat: @"[APP-LOVIN] Time out and video failed to load with error code %d", code];
        [PokktDebugger printLog:errorMsg];
    }
    else
    {
        [PokktDebugger printLog:errorMsg];
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:self.adConfig errorMessage:errorMsg];
    }
    self.screenName = nil;
    self.isCachingAvailable = YES;
}

#pragma mark - Ad Reward Delegate

- (void)rewardValidationRequestForAd:(ALAd *)ad didSucceedWithResponse:(nonnull NSDictionary *)response
{
    rewardAmount = [response objectForKey:@"amount"];
}

-(void) rewardValidationRequestForAd: (alnonnull ALAd*) ad didExceedQuotaWithResponse: (alnonnull NSDictionary*) response
{
    NSLog(@"%@",response);
}

-(void) rewardValidationRequestForAd: (alnonnull ALAd*) ad wasRejectedWithResponse: (alnonnull NSDictionary*) response
{
    NSLog(@"%@",response);
}

-(void) rewardValidationRequestForAd: (alnonnull ALAd*) ad didFailWithError: (NSInteger) responseCode
{
    NSLog(@"%ld",(long)responseCode);
}


#pragma mark - Ad Video Playback Delegate

- (void)videoPlaybackBeganInAd:(alnonnull ALAd *)ad
{
    [PokktDebugger printLog:@"[AppLovin SDK]Video Playback Began"];
    [PokktNetworkDelegate onAdDisplayed:self.network adConfig:self.adConfig];
}

- (void)videoPlaybackEndedInAd:(alnonnull ALAd *)ad atPlaybackPercent:(alnonnull NSNumber *)percentPlayed fullyWatched:(BOOL)wasFullyWatched
{
    if (wasFullyWatched)
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Video Ended"];
        [PokktNetworkDelegate onAdCompleted:self.network adConfig:self.adConfig];
        if (self.adConfig.isRewarded)
        {
            [PokktNetworkDelegate onAdGratified:self.network adConfig:self.adConfig rewardPoint:rewardAmount.floatValue];
        }
        
    }
    else
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Video Skipped"];
        [PokktNetworkDelegate onAdSkipped:self.network adConfig:self.adConfig];
    }
}

@end

@interface InterstitialAd()
@property(atomic)BOOL isCachingAvailable;
@property(atomic,strong) NSString *screenName;
@end


@implementation InterstitialAd

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.network = customData;
    self.isTimedout = NO;
    [ALInterstitialAd shared].adLoadDelegate = self;
    self.screenName = nil;
    self.isCachingAvailable = YES;
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if (self.screenName != nil && ![self.screenName isEqualToString:adConfig.screenName])
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Interstitial Caching failed.... Invalid network"];
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig
                                     errorMessage:@"Interstitial Caching Failed in AppLovin"];
        
        return;
    }
    
    if (self.isCachingAvailable)
    {
        self.isCachingAvailable = NO;
        self.adConfig = adConfig;
        self.screenName = adConfig.screenName;
        if ([ALInterstitialAd isReadyForDisplay])
        {
            [PokktDebugger printLog:@"[AppLovin SDK]Interstitial were already saved in cached"];
            [PokktNetworkDelegate didFinishedAdDownload:self.network adConfig:adConfig rewardValue:0 downLoadTime:@""];
        }
        else
        {
            [PokktDebugger printLog:@"[AppLovin SDK]Interstitial not available"];
            [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:@"[AppLovin SDK]Interstitial not available"];
        }
    }
    else
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Interstitial caching failed"];
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:@"[App Lovin] Invalid Network"];
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    self.adConfig = adConfig;
    
    if (self.screenName == nil || ![self.screenName isEqualToString:adConfig.screenName])
    {
        [PokktDebugger printLog:@"[AppLovin SDK]ScreenName cannot be nil to show interstitial Ad"];
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
        return;
    }

    if ([ALInterstitialAd isReadyForDisplay])
    {
        [ALInterstitialAd shared].adLoadDelegate = nil;
        [ALInterstitialAd shared].adDisplayDelegate = self;
        [ALInterstitialAd show];
    }
    else
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Failed to show interstitial Ad"];
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
    }
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    if (self.screenName == nil || ![self.screenName isEqualToString:adConfig.screenName])
    {
        [PokktDebugger printLog:@"[AdColony SDK]Ad available status is false.... due to nil screenname"];
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
        return;
    }

    BOOL flag = NO;
    if (!adConfig.isRewarded)
    {
        if ( [ALInterstitialAd isReadyForDisplay] )
        {
            flag = YES;
        }
    }
    [PokktNetworkDelegate onAdAvailabilityStatus:self.network adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    [PokktDebugger printLog:@"[AppLovin SDK]Caching time out"];
    self.isTimedout = YES;
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.network;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == 3)
    {
        flag = true;
    }
    
    return flag;
}

#pragma mark - Interstitial Ad Load Delegate

-(void) ad: (alnonnull ALAd *) ad wasDisplayedIn: (alnonnull UIView *) view
{
    [PokktDebugger printLog:@"[AppLovin SDK]Interstitial Loaded"];
    [PokktNetworkDelegate onAdDisplayed:self.network adConfig:self.adConfig];
}

-(void) ad: (alnonnull ALAd *) ad wasHiddenIn: (alnonnull UIView *) view
{
    [PokktDebugger printLog:@"[AppLovin SDK]Interstitial closed"];
    if (!self.adConfig.isRewarded) {
        self.isCachingAvailable = YES;
        self.screenName = nil;
    }
    
    [PokktNetworkDelegate onAdClosed:self.network adConfig:self.adConfig];
}

- (void)adService:(ALAdService *)adService didLoadAd:(ALAd *)ad
{
    if (self.isTimedout)
    {
        [PokktDebugger printLog:@"[AppLovin SDK]Interstitial timeOut"];
    }
    else
    {
         [PokktDebugger printLog:@"[AppLovin SDK]Interstitial downloaded"];
        [PokktNetworkDelegate didFinishedAdDownload:self.network adConfig:self.adConfig rewardValue:0 downLoadTime:@""];
    }
}

-(void) ad: (alnonnull ALAd *) ad wasClickedIn: (alnonnull UIView *) view
{
    [PokktDebugger printLog:@"[AppLovin SDK]Clicked on Interstitial"];
}

- (void)adService:(ALAdService *)adService didFailToLoadAdWithError:(int)code
{
    NSString *errorMsg = [NSString stringWithFormat: @"[APP-LOVIN]Interstitial failed to load with error code %d", code];
    if (self.isTimedout)
    {
        errorMsg = [NSString stringWithFormat: @"[APP-LOVIN] Time out and Interstitial failed to load with error code %d", code];
        [PokktDebugger printLog:errorMsg];
    }
    else
    {
        [PokktDebugger printLog:errorMsg];
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:self.adConfig errorMessage:errorMsg];
    }
    
    self.isCachingAvailable = YES;
    self.screenName = nil;
}

@end
