//
//  ChartboostNetwork.m
//  SampleApp
//
//  Created by Subhendu Sekhar Sahu on 08/10/15.
//  Copyright © 2015 Pokkt. All rights reserved.
//

#import "ChartboostNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"

@implementation ChartboostNetwork

#define versionNumber @"7.0.1"

static NSString *networkType = @"ChartBoost";

- (instancetype)init
{
    self = [super init];
    if (self)
    {
        self.reward  = 0.0f;
    }
    return self;
}

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    NSLog(@"%@ is initialised", networkType);
        
    self.networkModel  =customData;
    
    self.timedOutDictionary = [[NSMutableDictionary alloc]initWithCapacity:0];
    self.rewardedAdconfigDictionary = [[NSMutableDictionary alloc]initWithCapacity:0];
    self.nonRewardedAdconfigDictionary = [[NSMutableDictionary alloc] initWithCapacity:0];

    NSString * appId = [customData.customData objectForKey:@"appId"];
    NSString * appSignature = [customData.customData objectForKey:@"appSignature"];
    
    self.reward = [[customData.customData objectForKey:@"vc"] floatValue];
    
    [Chartboost startWithAppId:appId appSignature:appSignature delegate:self];
    
}


- (void) cacheAd: (PokktAdConfig *)adConfig
{
   if (adConfig.adFormat == VIDEO)
    {
        [self.rewardedAdconfigDictionary setObject:adConfig forKey:adConfig.screenName];
        
        [self.timedOutDictionary setObject:[NSNumber numberWithBool:NO] forKey:adConfig.getKey];
        
        if ([Chartboost hasRewardedVideo:adConfig.screenName])
        {
            NSLog(@"%@ video cached previously", networkType);
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:self.reward downLoadTime:@""];
        }
        else
        {
            NSLog(@"%@ video caching started", networkType);
            [Chartboost cacheRewardedVideo:adConfig.screenName];
        }
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.nonRewardedAdconfigDictionary setObject:adConfig forKey:adConfig.screenName];
        
        [self.timedOutDictionary setObject:[NSNumber numberWithBool:NO] forKey:adConfig.getKey];

        if ([Chartboost hasInterstitial:adConfig.screenName])
        {
            NSLog(@"%@ interstitial cached previously", networkType);
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
        }
        else
        {
            NSLog(@"%@ interstitial caching started", networkType);
            [Chartboost cacheInterstitial:adConfig.screenName];
        }
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (adConfig.adFormat == VIDEO)
    {
        [self.rewardedAdconfigDictionary setObject:adConfig forKey:adConfig.screenName];

        if ([Chartboost hasRewardedVideo:adConfig.screenName])
        {
            NSLog(@"%@ play video called", networkType);
            [Chartboost showRewardedVideo:adConfig.screenName];
        }
        else
        {
            [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        }
        
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.nonRewardedAdconfigDictionary setObject:adConfig forKey:adConfig.screenName];

        if ([Chartboost hasInterstitial:adConfig.screenName])
        {
            NSLog(@"%@ show Interstitial called", networkType);
            [Chartboost showInterstitial:adConfig.screenName];
        }
        else
        {
            [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        }
    }
}



-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;
    
    if (adConfig.adFormat == VIDEO && [Chartboost hasRewardedVideo:adConfig.screenName])
    {
       flag = YES;
    }
    else if (adConfig.adFormat == INTERSTITIAL && [Chartboost hasInterstitial:adConfig.screenName])
    {
        flag = YES;
    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:flag];
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag;
    
    if (adConfig.adFormat == VIDEO)
    {
        flag = true;
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        flag = true;
    }
    return flag;
}


#pragma mark -
#pragma mark - InterstitialMethod

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"[ChartBoost SDK] Caching time out");
    [self.timedOutDictionary setObject:[NSNumber numberWithBool:YES] forKey:adConfig.getKey];
}

#pragma mark -
#pragma mark - Chartboost delegates
#pragma mark -


/*
 * Chartboost Delegate Methods
 *
 */
- (BOOL)shouldDisplayRewardedVideo:(CBLocation)location;
{
    return TRUE;
}


- (void)didCacheRewardedVideo:(CBLocation)location
{
    PokktAdConfig * adConfig = [self.rewardedAdconfigDictionary objectForKey:location];
    
    if (adConfig != nil)
    {
        BOOL isTimedout = [[self.timedOutDictionary objectForKey:adConfig.getKey] boolValue];
        
        if (isTimedout)
        {
            NSLog(@"%@ video caching successful after timeout", networkType);
        }
        else
        {
            NSLog(@"%@ video caching successful before timeout", networkType);
            
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:self.reward downLoadTime:@""];
        }

    }
    else
    {
        NSLog(@"didCacheRewarded() : Location not defined");
    }
}

/*
 * didCompleteRewardedVideo
 *
 * This is called when a rewarded video has been viewed
 *
 * Is fired on:
 * - Rewarded video completed view
 *
 */

- (void)didCompleteRewardedVideo:(CBLocation)location withReward:(int)reward
{
    NSLog(@"%@ video completed", networkType);
    
    NSLog(@"completed rewarded video view at location %@ with reward amount %d", location, reward);
    
    PokktAdConfig * adConfig = [self.rewardedAdconfigDictionary objectForKey:location];
    
    if (adConfig != nil)
    {
        [PokktNetworkDelegate onAdCompleted:self.networkModel adConfig:adConfig];

        if (adConfig.isRewarded)
        {
            [PokktNetworkDelegate onAdGratified:self.networkModel adConfig:adConfig rewardPoint:reward];
        }
        
    }
    else
    {
        NSLog(@"didCompleteRewardedVideo() : Location not defined");
    }
}

/*
 * didFailToLoadRewardedVideo
 *
 * This is called when a Rewarded Video has failed to load. The error enum specifies
 * the reason of the failure
 */

- (void)didFailToLoadRewardedVideo:(NSString *)location withError:(CBLoadError)error
{
    NSString *errorMsg;
    switch(error){
        case CBLoadErrorInternetUnavailable:
        {
            errorMsg = @"Failed to load Rewarded Video, no Internet connection !";
        } break;
        case CBLoadErrorInternal: {
            errorMsg = @"Failed to load Rewarded Video, internal error !";
        } break;
        case CBLoadErrorNetworkFailure: {
            errorMsg = @"Failed to load Rewarded Video, network error !";
        } break;
        case CBLoadErrorWrongOrientation: {
            errorMsg = @"Failed to load Rewarded Video, wrong orientation !";
        } break;
        case CBLoadErrorTooManyConnections: {
            errorMsg = @"Failed to load Rewarded Video, too many connections !";
        } break;
        case CBLoadErrorFirstSessionInterstitialsDisabled: {
            errorMsg = @"Failed to load Rewarded Video, first session !";
        } break;
        case CBLoadErrorNoAdFound : {
            errorMsg = @"Failed to load Rewarded Video, no ad found !";
        } break;
        case CBLoadErrorSessionNotStarted : {
            errorMsg = @"Failed to load Rewarded Video, session not started !";
        } break;
        case CBLoadErrorNoLocationFound : {
            errorMsg = @"Failed to load Rewarded Video, missing location parameter !";
        } break;
        default: {
            errorMsg = @"Failed to load Rewarded Video, unknown error !";
        }
    }
    
    PokktAdConfig * adConfig = [self.rewardedAdconfigDictionary objectForKey:location];
    
    if (adConfig != nil)
    {
        BOOL isTimedout = [[self.timedOutDictionary objectForKey:adConfig.getKey] boolValue];
        
        if (isTimedout)
        {
            NSLog(@"%@ video caching failed after timeout", networkType);
        }
        else
        {
            NSLog(@"%@ video caching failed before timeout", networkType);
            
            [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:errorMsg];
        }
        
    }
    else
    {
        NSLog(@"didFailToLoadRewardedVideo() : Location not defined");
    }
}

/*!
 @abstract
 Called after a InPlay has attempted to load from the Chartboost API
 servers but failed.
 
 @param location The location for the Chartboost impression type.
 
 @param error The reason for the error defined via a CBLoadError.
 
 @discussion Implement to be notified of when an InPlay has attempted to load from the Chartboost API
 servers but failed for a given CBLocation.
 */
- (void)didFailToLoadInPlay:(CBLocation)location
                  withError:(CBLoadError)error
{
    
    NSString *errorString = @"";
    switch(error){
        case CBLoadErrorInternetUnavailable: {
            errorString = @"Failed to load In Play, no Internet connection !";
        } break;
        case CBLoadErrorInternal: {
            errorString = @"Failed to load In Play, internal error !";
        } break;
        case CBLoadErrorNetworkFailure: {
            errorString = @"Failed to load In Play, network errorString !";
        } break;
        case CBLoadErrorWrongOrientation: {
            errorString = @"Failed to load In Interstitial, wrong orientation !";
        } break;
        case CBLoadErrorTooManyConnections: {
            errorString = @"Failed to load In Play, too many connections !";
        } break;
        case CBLoadErrorFirstSessionInterstitialsDisabled: {
            errorString = @"Failed to load In Play, first session !";
        } break;
        case CBLoadErrorNoAdFound : {
            errorString = @"Failed to load In Play, no ad found !";
        } break;
        case CBLoadErrorSessionNotStarted : {
            errorString = @"Failed to load In Play, session not started !";
        } break;
        case CBLoadErrorNoLocationFound : {
            errorString = @"Failed to load In Play, missing location parameter !";
        } break;
        default: {
            errorString = @"Failed to load In Play, unknown error !";
        }
    }
    
    NSLog(@"%@ video download failed", networkType);
    
    PokktAdConfig * adConfig = [self.rewardedAdconfigDictionary objectForKey:location];
    
    if (adConfig != nil)
    {
        BOOL isTimedout = [[self.timedOutDictionary objectForKey:adConfig.getKey] boolValue];
        
        if (isTimedout)
        {
            NSLog(@"%@ video caching failed after timeout", networkType);
        }
        else
        {
            NSLog(@"%@ video caching failed before timeout", networkType);
            
            [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:errorString];
        }
    }
    else
    {
        NSLog(@"didFailToLoadInPlay() : Location not defined");
    }
}

- (void)didCloseRewardedVideo:(CBLocation)location
{
    PokktAdConfig * adConfig = [self.rewardedAdconfigDictionary objectForKey:location];
    
    if (adConfig != nil)
    {
        [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:adConfig];
    }
    else
    {
        NSLog(@"didCloseRewardedVideo() : Location not defined");
    }
}

- (void)didDisplayRewardedVideo:(CBLocation)location
{
    PokktAdConfig * adConfig = [self.rewardedAdconfigDictionary objectForKey:location];
    
    if (adConfig != nil)
    {
        [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:adConfig];
    }
    else
    {
        NSLog(@"didDisplayRewardedVideo() : Location not defined");
    }
}

#pragma mark -
#pragma mark - Interstitial Delegate Methods

- (void)didCacheInterstitial:(CBLocation)location
{
    PokktAdConfig * adConfig = [self.nonRewardedAdconfigDictionary objectForKey:location];
    
    if (adConfig != nil)
    {
        BOOL isTimedout = [[self.timedOutDictionary objectForKey:adConfig.getKey] boolValue];
        
        if (isTimedout)
        {
            NSLog(@"%@ interstitial caching successful after timeout", networkType);
        }
        else
        {
            NSLog(@"%@ interstitial caching successful before timeout", networkType);
            
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
        }
        
    }
    else
    {
        NSLog(@"didCacheInterstitial() : Location not defined");
    }
}

- (void)didDisplayInterstitial:(CBLocation)location
{
    PokktAdConfig * adConfig = [self.nonRewardedAdconfigDictionary objectForKey:location];
    
    if (adConfig != nil)
    {
        [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:adConfig];
    }
    else
    {
        NSLog(@"didDisplayInterstitial() : Location not defined");
    }
}

- (void)didFailToLoadInterstitial:(CBLocation)location
                        withError:(CBLoadError)error
{
    NSString *errorMsg;
    switch(error){
        case CBLoadErrorInternetUnavailable: {
            errorMsg = @"Failed to load In Interstitial, no Internet connection !";
        } break;
        case CBLoadErrorInternal: {
            errorMsg = @"Failed to load In Interstitial, internal error !";
        } break;
        case CBLoadErrorNetworkFailure: {
            errorMsg = @"Failed to load In Interstitial, network errorString !";
        } break;
        case CBLoadErrorWrongOrientation: {
            errorMsg = @"Failed to load In Interstitial, wrong orientation !";
        } break;
        case CBLoadErrorTooManyConnections: {
            errorMsg = @"Failed to load In Interstitial, too many connections !";
        } break;
        case CBLoadErrorFirstSessionInterstitialsDisabled: {
            errorMsg = @"Failed to load In Interstitial, first session !";
        } break;
        case CBLoadErrorNoAdFound : {
            errorMsg = @"Failed to load In Interstitial, no ad found !";
        } break;
        case CBLoadErrorSessionNotStarted : {
            errorMsg = @"Failed to load In Interstitial, session not started !";
        } break;
        case CBLoadErrorNoLocationFound : {
            errorMsg = @"Failed to load In Interstitial, missing location parameter !";
        } break;
        default: {
            errorMsg = @"Failed to load In Play, unknown error !";
        }
    }
    
    PokktAdConfig * adConfig = [self.nonRewardedAdconfigDictionary objectForKey:location];
    
    if (adConfig != nil)
    {
        BOOL isTimedout = [[self.timedOutDictionary objectForKey:adConfig.getKey] boolValue];
        
        if (isTimedout)
        {
            NSLog(@"%@ interstitial caching failed after timeout", networkType);
        }
        else
        {
            NSLog(@"%@ interstitial caching failed before timeout", networkType);
            
            [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:errorMsg];
        }
    }
    else
    {
        NSLog(@"didFailToLoadInterstitial() : Location not defined");
    }
}

- (void)didFailToRecordClick:(CBLocation)location
                   withError:(CBClickError)error
{
    
}

- (void)didDismissInterstitial:(CBLocation)location
{
    PokktAdConfig * adConfig = [self.nonRewardedAdconfigDictionary objectForKey:location];
    
    if (adConfig != nil)
    {
        [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:adConfig];
    }
    else
    {
        NSLog(@"didCloseInterstitial() : Location not defined");
    }
}

- (void)didCloseInterstitial:(CBLocation)location
{
    PokktAdConfig * adConfig = [self.nonRewardedAdconfigDictionary objectForKey:location];

    if (adConfig != nil)
    {
        [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:adConfig];
    }
    else
    {
        NSLog(@"didCloseInterstitial() : Location not defined");
    }
}

- (void)didClickInterstitial:(CBLocation)location
{
    
}

@end
