//
//  FacebookNetwork.m
//  SampleApp
//
//  Created by Subhendu Sekhar Sahu on 29/01/16.
//  Copyright © 2016 Pokkt. All rights reserved.
//

#import "FacebookNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"

#define versionNumber @"4.18.1"
#define logMessage(message) [NSString stringWithFormat:@"FBM: %@",message]

@implementation FacebookNetwork

static BOOL isInitialized;

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    if (isInitialized)
    {
        [PokktDebugger printLog:logMessage(@"Already Initialised Facebook Network")];
        return;
    }

    isInitialized = YES;

    self.networkModel  =customData;
    self.interstitialAd = [[facebookInterstitial alloc] init];
    [self.interstitialAd initNetworkWithNetworkModel:customData];
    
    self.bannerAd = [[facebookBanner alloc] init];
    self.bannerAd.networkModel = customData;
    
    
}

-(void) fetchAd:(PokktAdConfig *)adConfig withAdView:(UIView *)bannerView withRootViewController:(UIViewController *)rootViewController withDelegate:(id<PokktBannerDelegate>)bannerDelegate onSuccess:(void (^)(id))successCallback onFailure:(void (^)(id))failureHandler
{
    [self.bannerAd fetchAd:adConfig withAdView:bannerView withRootViewController:rootViewController withDelegate:bannerDelegate onSuccess:successCallback onFailure:failureHandler];
}


- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.interstitialAd cacheAd:adConfig];
    }
    
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.interstitialAd showAd:adConfig viewController:viewController];
    }
    
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.interstitialAd checkAdAvailability:adConfig];
    }
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}


- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.interstitialAd setCacheTimedOut:adConfig];
    }
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        return [self.interstitialAd checkAdFormatSupport:adConfig];
    }
    else if (adConfig.adFormat == BANNER)
    {
        return [self.bannerAd checkAdFormatSupport:adConfig];
    }
    
    return false;
}


@end

@implementation facebookInterstitial

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    
    mappingDict = [[NSMutableDictionary alloc] initWithCapacity:0];
    adConfigDict = [[NSMutableDictionary alloc] initWithCapacity:0];
    adMap = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.networkModel = customData;
    NSString  *testDeviceID = [customData.customData objectForKey:@"TestDeviceId"];
    
    if (testDeviceID != nil && testDeviceID.length > 0)
    {
        NSArray *deviceIdArr = [testDeviceID componentsSeparatedByString:@","];
       // [FBAdSettings addTestDevices:deviceIdArr];
    }
    if ([PokktDebugger isDebugEnabled])
    {
        [FBAdSettings setLogLevel:FBAdLogLevelLog];
    }
 
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if (adConfig.isRewarded)
    {
        [PokktDebugger printLog:logMessage(@"Caching failed.... Invalid network")];
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"[Facebook SDK] Invalid network"];
        return;
    }
    
    NSString *screenName = [self getScreenName:adConfig];
    
    if (screenName == nil && screenName.length == 0)
    {
        [PokktDebugger printLog:logMessage(@"Caching failed.... ScreenName cannot be nil to cacheAd")];
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"[Facebook SDK] Invalid network"];
        return;
    }
    
    [self cachingAdWithScreenName:screenName PokktAdConfig:adConfig];
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (adConfig.isRewarded)
    {
        [PokktDebugger printLog:logMessage(@"Failed to show ad...Invalid Network")];
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        return;
    }
    
    NSString *screenName = [self getScreenName:adConfig];
    
    if (screenName == nil && screenName.length == 0 && [screenName isEqualToString:@""])
    {
        [PokktDebugger printLog:logMessage(@"Failed to show ad...ScreenName cannot be nil for showAd")];
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        return;
    }
    
    self.viewController = viewController;
    
    NSString *rewardKey = [self getRewardedKey:adConfig.isRewarded];
    
    if ([[[mappingDict objectForKey:screenName] objectForKey:rewardKey] boolValue])
    {
        if ([adMap objectForKey:screenName])
        {
            [adConfigDict setObject:adConfig forKey:screenName];
            FBInterstitialAd *interstitialAd = (FBInterstitialAd *)[adMap objectForKey:screenName];
            [interstitialAd showAdFromRootViewController:self.viewController];
        }
        else
        {
            [PokktDebugger printLog:logMessage(@"Failed to show ad...ScreenName not available in Map")];
            [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        }
    }
    else
    {
        [PokktDebugger printLog:logMessage(@"Failed to show ad...Invalid Network")];
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }
 
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    if (adConfig.isRewarded)
    {
        [PokktDebugger printLog:logMessage(@"Ad available status is false...Invalid Network")];
        [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:NO];
        return;
    }
    
    NSString *screenName = [self getScreenName:adConfig];
    
    if (screenName == nil && screenName.length == 0)
    {
        [PokktDebugger printLog:logMessage(@"Ad available status is false...due to nil screenname")];
        [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:NO];
        return;
    }
    
    NSString *rewardKey = [self getRewardedKey:adConfig.isRewarded];
    
    if ([[[mappingDict objectForKey:screenName] objectForKey:rewardKey] boolValue])
    {
        if ([adMap objectForKey:screenName])
        {
            [PokktDebugger printLog:logMessage(@"Ad available status is true...")];
            [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:YES];
        }
        else
        {
            [PokktDebugger printLog:logMessage(@"Ad available status is false...invalid screenname")];
            [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:NO];
        }
        
    }
    else
    {
        [PokktDebugger printLog:logMessage(@"Ad available status is false...invalid screenname")];
        [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:NO];
    }
 
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    [PokktDebugger printLog:logMessage(@"Caching time out")];
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == 3)
    {
        return true;
    }
    
    return false;
}

-(void)cachingAdWithScreenName:(NSString *)screen PokktAdConfig:(PokktAdConfig *)adConfig
{
    [self setAdAvailableForScreenName:screen isRewarded:adConfig.isRewarded];
    
    FBInterstitialAd *interstitialAd = [[FBInterstitialAd alloc] initWithPlacementID:screen];
    interstitialAd.delegate = self;
    [interstitialAd loadAd];
    
    [adConfigDict setObject:adConfig forKey:screen];
    [adMap setObject:interstitialAd forKey:screen];
}

-(NSString *)getScreenName:(PokktAdConfig *)adConfig
{
    NSString *screenName = nil;
    
    if ([self.networkModel.customData objectForKey:@"screens_mapping_data"] != nil && [self.networkModel.customData objectForKey:@"screens_mapping_data"])
    {
        NSDictionary *screenDict = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
        
        if ([screenDict objectForKey:adConfig.screenName] != nil && [screenDict objectForKey:adConfig.screenName])
        {
            NSDictionary *networkDict = [screenDict objectForKey:adConfig.screenName];
            
            if ([networkDict objectForKey:@"network_screen"] != nil && [networkDict objectForKey:@"network_screen"])
            {
                screenName = [networkDict objectForKey:@"network_screen"];
            }
        }
    }
    else
    {
        screenName = adConfig.screenName;
    }
    
    return screenName;
}

- (void)interstitialAdDidLoad:(FBInterstitialAd *)interstitialAd
{
    [PokktDebugger printLog:logMessage(@"Ad is loaded and ready to be displayed")];
    PokktAdConfig *adConfigue = (PokktAdConfig *)[adConfigDict objectForKey:interstitialAd.placementID];
    [self setAdAvailableForScreenName:interstitialAd.placementID isRewarded:adConfigue.isRewarded];
    
    if (adConfigue != nil)
    {
        [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfigue rewardValue:0 downLoadTime:@""];
    }
    
}

- (void)interstitialAd:(FBInterstitialAd *)interstitialAd didFailWithError:(NSError *)error
{
    [PokktDebugger printLog:logMessage(@"Ad failed to load")];
    PokktAdConfig *adConfigue = (PokktAdConfig *)[adConfigDict objectForKey:interstitialAd.placementID];
    
    if (adConfigue != nil) {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfigue errorMessage:@"[Facebook SDK]Interstitial Ad is not available"];
    }
    
}

- (void)interstitialAdDidClose:(FBInterstitialAd *)interstitialAd
{
    [PokktDebugger printLog:logMessage(@"Ad did closed")];
    PokktAdConfig *adConfigue = (PokktAdConfig *)[adConfigDict objectForKey:interstitialAd.placementID];
    
    if (adConfigue != nil) {
        [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:adConfigue];
    }
    [mappingDict removeObjectForKey:interstitialAd.placementID];
    [adMap removeObjectForKey:interstitialAd.placementID];
}


- (void)interstitialAdDidClick:(FBInterstitialAd *)interstitialAd
{
    [PokktDebugger printLog:logMessage(@"Ad clicked")];
}

- (void)interstitialAdWillClose:(FBInterstitialAd *)interstitialAd
{
    [PokktDebugger printLog:logMessage(@"Ad will closed")];
}

- (void)interstitialAdWillLogImpression:(FBInterstitialAd *)interstitialAd
{
    [PokktDebugger printLog:logMessage(@"Ad impression is being captured.")];
}

-(NSMutableDictionary *) setAdAvailableForScreenName:(NSString *)screenName isRewarded:(BOOL)isRewarded
{
    NSString *str = @"";
    
    if (isRewarded) {
        str = @"Rewarded";
    }
    else
    {
        str = @"NonRewarded";
    }
    if (![mappingDict objectForKey:screenName]) {
        NSMutableDictionary *rewardDict = [[NSMutableDictionary alloc] initWithCapacity:0];
        [rewardDict setObject:[NSNumber numberWithInt:0] forKey:str];
        [mappingDict setObject:rewardDict forKey:screenName];
    }
    else
    {
        NSMutableDictionary *dict = (NSMutableDictionary *)[mappingDict objectForKey:screenName];
        if ([dict objectForKey:str])
        {
            [dict setObject:[NSNumber numberWithInt:1] forKey:str];
        }
        else
        {
            [dict setObject:[NSNumber numberWithInt:0] forKey:str];
        }
    }
    
    return mappingDict;
}

-(NSString *)getRewardedKey:(BOOL)isRewarded
{
    NSString *str = @"";
    
    if (isRewarded) {
        str = @"Rewarded";
    }
    else
    {
        str = @"NonRewarded";
    }
    return str;
}

@end

@implementation facebookBanner

-(void) fetchAd:(PokktAdConfig *)adConfig withAdView:(UIView *)bannerView withRootViewController:(UIViewController *)rootViewController withDelegate:(id<PokktBannerDelegate>)bannerDelegate onSuccess:(void (^)(id))successCallback onFailure:(void (^)(id))failureHandler
{

    NSString *screenName = [self getScreenName:adConfig];
    self.bannerDelegate = bannerDelegate;
    if (screenName != nil && screenName.length != 0)
    {
    FBAdView *adView = [[FBAdView alloc] initWithPlacementID:screenName
                                                      adSize:kFBAdSize320x50
                                          rootViewController:rootViewController];
    adView.delegate = self;
    
    if ([self.networkModel.customData objectForKey:@"TestDeviceId"])
    {
        [FBAdSettings addTestDevice:[self.networkModel.customData objectForKey:@"TestDeviceId"]];
    }
    
    [adView loadAd];
    CGSize size = bannerView.bounds.size;
    adView.frame = CGRectMake(0, 0, size.width, size.height);
    
    [bannerView addSubview:adView];
    }
    else
    {
        [bannerDelegate bannerLoadFailed:screenName errorMessage:@"Screename is not valid"];
    }
    
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == 1)
    {
        return true;
    }
    
    return false;
}

- (void)adView:(FBAdView *)adView didFailWithError:(NSError *)error;
{
    [self.bannerDelegate bannerLoadFailed:@"" errorMessage:error.description];
}

- (void)adViewDidLoad:(FBAdView *)adView;
{
    [self.bannerDelegate bannerLoaded:@""];
}

-(NSString *)getScreenName:(PokktAdConfig *)adConfig
{
    NSString *screenName = nil;
    
    if ([self.networkModel.customData objectForKey:@"screens_mapping_data"] != nil && [self.networkModel.customData objectForKey:@"screens_mapping_data"])
    {
        NSDictionary *screenDict = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
        
        if ([screenDict objectForKey:adConfig.screenName] != nil && [screenDict objectForKey:adConfig.screenName])
        {
            NSDictionary *networkDict = [screenDict objectForKey:adConfig.screenName];
            
            if ([networkDict objectForKey:@"network_screen"] != nil && [networkDict objectForKey:@"network_screen"])
            {
                screenName = [networkDict objectForKey:@"network_screen"];
            }
        }
    }
    else
    {
        screenName = adConfig.screenName;
    }
    
    return screenName;//@"1247645121954713_1247646215287937";
}



@end
