//
//  FlurryNetwork.m
//  SampleApp
//
//  Created by Pokkt on 21/11/16.
//  Copyright © 2016 Pokkt. All rights reserved.
//

#import "FlurryNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"

#define versionNumber @"8.2.1"

@implementation FlurryNetwork

static BOOL isInitialized;

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    if (isInitialized) {
        return;
    }
    isInitialized = YES;
    self.networkModel = customData;
    
    self.interstitialAd = [[FlurryInterstitial alloc] init];
    [self.interstitialAd initNetworkWithNetworkModel:customData];
    
    self.bannerAd = [[FlurryBanner alloc] init];
    self.bannerAd.networkModel = customData;
   
    
    NSLog(@"[Flurry SDK]Flurry Initialised");
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    [self.interstitialAd cacheAd:adConfig];
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
   [self.interstitialAd showAd:adConfig viewController:viewController];
}

-(void) fetchAd:(PokktAdConfig *)adConfig withAdView:(UIView *)bannerView withRootViewController:(UIViewController *)rootViewController withDelegate:(id<PokktBannerDelegate>)bannerDelegate onSuccess:(void (^)(id))successCallback onFailure:(void (^)(id))failureHandler
{
    [self.bannerAd fetchAd:adConfig withAdView:bannerView withRootViewController:rootViewController withDelegate:bannerDelegate onSuccess:successCallback onFailure:failureHandler];
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    [self.interstitialAd checkAdAvailability:adConfig];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    [self.interstitialAd setCacheTimedOut:adConfig];
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == BANNER)
    {
        return [self.bannerAd checkAdFormatSupport:adConfig];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        return [self.interstitialAd checkAdFormatSupport:adConfig];
    }
    
    return false;
}

@end

@implementation FlurryInterstitial

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
 
    self.networkModel = customData;
    
    self.interstitialMap = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.adConfigDictionary = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.timedOutDictionary = [[NSMutableDictionary alloc] initWithCapacity:0];
    
    NSString *appKey = [self.networkModel.customData objectForKey:@"ApiKey"];
    if (appKey != nil && appKey.length > 0)
    {
        [Flurry setLogLevel:FlurryLogLevelDebug];
        [Flurry startSession:appKey];
        [PokktDebugger printLog:@"[Flurry SDK]Flurry Initialised"];
    }
    else
    {
        [PokktDebugger printLog:@"[Flurry SDK] Invalid ApiKey... Initialisation failed."];
    }
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
     NSString *screenName = [self getScreenName:adConfig];
    if (screenName != nil && screenName.length > 0)
    {
        [self.adConfigDictionary setObject:adConfig forKey:adConfig.screenName];
        FlurryAdInterstitial *interstitialAd = [self getFlurryInterstitialAd:screenName];
       
        if (interstitialAd != nil)
        {
            if ([interstitialAd ready])
            {
                [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0.0 downLoadTime:@""];
            }
        }
        else
        {
            [PokktDebugger printLog:@"[Flurry SDK]Caching failed for interstitial ad.... Invalid network"];
            [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"Caching failed for interstitial ad in Flurry"];
        }
    }
    else
    {
        [PokktDebugger printLog:@"[Flurry SDK]Caching failed for interstitial ad.... Invalid network"];
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"Caching failed for interstitial ad in Flurry"];
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (viewController == nil)
    {
        viewController = [[[UIApplication sharedApplication] keyWindow] rootViewController];
    }
    
    NSString *screenName = [self getScreenName:adConfig];
    if (screenName != nil && screenName.length > 0)
    {
        [self.adConfigDictionary setObject:adConfig forKey:adConfig.screenName];
        FlurryAdInterstitial *interstitialAd = [self getFlurryInterstitialAd:screenName];
        
        if ([interstitialAd ready])
        {
            [interstitialAd presentWithViewController:viewController];
        }
        else
        {
            [PokktDebugger printLog:@"[Flurry SDK] Failed to show ad...Ad is not ready to show"];
            [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        }
    }
    else
    {
        [PokktDebugger printLog:@"[Flurry SDK] Failed to show ad...invalid network"];
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }
    
   
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;
    NSString *screenName = [self getScreenName:adConfig];
    if (screenName != nil && screenName.length > 0)
    {
        [self.adConfigDictionary setObject:adConfig forKey:adConfig.screenName];
        FlurryAdInterstitial *interstitialAd = [self getFlurryInterstitialAd:screenName];
        
        if ([interstitialAd ready])
        {
            flag = YES;
        }
    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"[Flurry SDK] Caching time out");
     [self.timedOutDictionary setObject:[NSNumber numberWithBool:YES] forKey:adConfig.getKey];
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
         return true;
    }
    return false;
}

-(FlurryAdInterstitial *) getFlurryInterstitialAd:(NSString *)spaceName
{
    FlurryAdInterstitial *adInterstitial = nil;
    if ([self.interstitialMap objectForKey:spaceName] != nil && [self.interstitialMap objectForKey:spaceName])
    {
        adInterstitial = [self.interstitialMap objectForKey:spaceName];
    }
    else
    {
        adInterstitial = [[FlurryAdInterstitial alloc]  initWithSpace:spaceName];
        
        adInterstitial.adDelegate = self;
        [adInterstitial fetchAd];
        [self.interstitialMap setObject:adInterstitial forKey:spaceName];
    }
    
    return adInterstitial;
}

-(NSString *)getScreenName:(PokktAdConfig *)adConfig
{
    NSString *screenName = @"";
    
    if ([self.networkModel.customData objectForKey:@"screens_mapping_data"] != nil && [self.networkModel.customData objectForKey:@"screens_mapping_data"])
    {
        NSDictionary *screenDict = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
        
        if ([screenDict objectForKey:adConfig.screenName] != nil && [screenDict objectForKey:adConfig.screenName])
        {
            NSDictionary *networkDict = [screenDict objectForKey:adConfig.screenName];
            
            if ([networkDict objectForKey:@"network_screen"] != nil && [networkDict objectForKey:@"network_screen"])
            {
                screenName = [networkDict objectForKey:@"network_screen"];
            }
        }
    }
    else
    {
        screenName = adConfig.screenName;
    }
    
    return screenName;
}

- (void) adInterstitialDidFetchAd:(FlurryAdInterstitial*)interstitialAd
{
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:interstitialAd.space];
    if (adConfig != nil)
    {
        BOOL timeOut = [[self.timedOutDictionary objectForKey:adConfig.getKey] boolValue];
        if (timeOut)
        {
            [PokktDebugger printLog:@"[Flurry SDK] Caching timeout..."];
        }
        else
        {
            [PokktDebugger printLog:@"[Flurry SDK] Caching successfully"];
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0.0 downLoadTime:@""];
        }
    }
}

- (void) adInterstitialDidRender:(FlurryAdInterstitial*)interstitialAd
{
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:interstitialAd.space];
    [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:adConfig];
}

- (void) adInterstitialWillPresent:(FlurryAdInterstitial*)interstitialAd
{
    
}

- (void) adInterstitialWillLeaveApplication:(FlurryAdInterstitial*)interstitialAd
{
    
}

- (void) adInterstitialWillDismiss:(FlurryAdInterstitial*)interstitialAd
{
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:interstitialAd.space];
    [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:adConfig];
}

- (void) adInterstitialDidDismiss:(FlurryAdInterstitial*)interstitialAd
{
    
}

- (void) adInterstitialDidReceiveClick:(FlurryAdInterstitial*)interstitialAd
{
    
}

- (void) adInterstitialVideoDidFinish:(FlurryAdInterstitial*)interstitialAd
{
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:interstitialAd.space];
    [PokktNetworkDelegate onAdCompleted:self.networkModel adConfig:adConfig];
}

- (void) adInterstitial:(FlurryAdInterstitial*) interstitialAd adError:(FlurryAdError) adError errorDescription:(NSError*) errorDescription
{
    [PokktDebugger printLog:@"[Flurry SDK]Caching failed for interstitial ad.... Invalid network"];
    
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:interstitialAd.space];
    [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"Caching failed for interstitial ad in Flurry"];
}

@end

@implementation FlurryBanner

-(void) fetchAd:(PokktAdConfig *)adConfig withAdView:(UIView *)bannerView withRootViewController:(UIViewController *)rootViewController withDelegate:(id<PokktBannerDelegate>)bannerDelegate onSuccess:(void (^)(id))successCallback onFailure:(void (^)(id))failureHandler
{
    //FLURRY_BANNER_AD
    NSString *screenName = [self getScreenName:adConfig];
    self.bannerDelegate = bannerDelegate;
    if (screenName != nil && screenName.length != 0)
    {
        FlurryAdBanner *adBanner = [[FlurryAdBanner alloc] initWithSpace:screenName];
        adBanner.adDelegate = self;
        
        [adBanner fetchAndDisplayAdInView:bannerView viewControllerForPresentation:rootViewController];
        
    }
    else
    {
      [self.bannerDelegate bannerLoadFailed:screenName errorMessage:@"[Flurry SDK] Invalid network"];
    }
    
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == BANNER)
    {
        return true;
    }
    return false;
}

-(NSString *)getScreenName:(PokktAdConfig *)adConfig
{
    NSString *screenName = @"";
    
    if ([self.networkModel.customData objectForKey:@"screens_mapping_data"] != nil && [self.networkModel.customData objectForKey:@"screens_mapping_data"])
    {
        NSDictionary *screenDict = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
        
        if ([screenDict objectForKey:adConfig.screenName] != nil && [screenDict objectForKey:adConfig.screenName])
        {
            NSDictionary *networkDict = [screenDict objectForKey:adConfig.screenName];
            
            if ([networkDict objectForKey:@"network_screen"] != nil && [networkDict objectForKey:@"network_screen"])
            {
                screenName = [networkDict objectForKey:@"network_screen"];
            }
        }
    }
    else
    {
        screenName = adConfig.screenName;
    }
    
    return screenName; 
}


- (void) adBannerDidFetchAd:(FlurryAdBanner *) adBanner
{
   [self.bannerDelegate bannerLoaded:adBanner.space];
}

- (void) adBanner:(FlurryAdBanner*) bannerAd adError:(FlurryAdError) adError errorDescription:(NSError*) errorDescription
{
    [self.bannerDelegate bannerLoadFailed:bannerAd.space errorMessage:errorDescription.description];
}

- (void) adBannerDidRender:(FlurryAdBanner*)bannerAd
{
    
}

- (void) adBannerWillPresentFullscreen:(FlurryAdBanner*)bannerAd
{
    
}

- (void) adBannerWillLeaveApplication:(FlurryAdBanner*)bannerAd
{
    
}

- (void) adBannerWillDismissFullscreen:(FlurryAdBanner*)bannerAd
{
    
}

- (void) adBannerDidDismissFullscreen:(FlurryAdBanner*)bannerAd
{
    
}

- (void) adBannerDidReceiveClick:(FlurryAdBanner*)bannerAd
{
    
}

- (void) adBannerVideoDidFinish:(FlurryAdBanner*)bannerAd
{
    
}


@end
