//
//  FyberNetwork.m
//  SampleApp
//
//  Created by Pokkt on 13/11/15.
//  Copyright © 2015 Pokkt. All rights reserved.
//

#import "FyberNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"

#define versionNumber @"8.15.1"

@implementation FyberNetwork

@synthesize networkName;

static bool isInitialized;
static NSString *token;

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    if (isInitialized)
    {
        return;
    }
    
    NSString *appId = [customData.customData objectForKey:@"AppKey"];
    token = [customData.customData objectForKey:@"security_token"];
    self.networkModel = customData;
    
    if ([PokktDebugger isDebugEnabled])
    {
        [FyberSDK setLoggingLevel:FYBLogLevelDebug];
    }
    
    if (appId != nil && appId.length > 0 && token != nil && token.length > 0)
    {
        
        FYBSDKOptions *options = [FYBSDKOptions optionsWithAppId:appId securityToken:token];
        options.startVideoPrecaching = YES;
        [FyberSDK startWithOptions:options];
        
        isInitialized = YES;
        NSLog(@"[Fyber SDK]Fyber Initialised");
        
        self.videoFyberNetwork = [[VideoFyberNetwork alloc ] init];
        [self.videoFyberNetwork initNetworkWithNetworkModel:customData];
        
        self.interstitialFyberNetwork = [[InterstitialFyberNetwork alloc] init];
        [self.interstitialFyberNetwork initNetworkWithNetworkModel:customData];
    }
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
        [self.videoFyberNetwork cacheAd:adConfig];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.interstitialFyberNetwork cacheAd:adConfig];
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (adConfig.adFormat == VIDEO)
    {
        [self.videoFyberNetwork showAd:adConfig viewController:viewController];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.interstitialFyberNetwork showAd:adConfig viewController:viewController];
    }
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
       return [self.videoFyberNetwork checkAdFormatSupport:adConfig];
    }
    
    else if (adConfig.adFormat == INTERSTITIAL)
    {
       return [self.interstitialFyberNetwork checkAdFormatSupport:adConfig];
    }

    return false;
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
        [self.videoFyberNetwork checkAdAvailability:adConfig];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.interstitialFyberNetwork checkAdAvailability:adConfig];
    }
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
        [self.videoFyberNetwork setCacheTimedOut:adConfig];
    }
    else if (adConfig.adFormat == VIDEO)
    {
        [self.interstitialFyberNetwork setCacheTimedOut:adConfig];
    }
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

@end

@implementation VideoFyberNetwork

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.network = customData;
    self.cacheLock = [NSRecursiveLock new];
    self.isVideoCached = NO;
    self.isTimedout = NO;
}

- (void)cacheAd:(PokktAdConfig *)adConfig
{
    if ([self.cacheLock tryLock])
    {
        NSString *str = nil;
        
        if (self.currentScreenName != nil && ![self.currentScreenName isEqualToString:adConfig.screenName])
        {
            [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig
                                         errorMessage:@"Caching in Fyber Failed"];
            
            return;
        }
        
        self.currentScreenName = adConfig.screenName;
        
        if (self.isVideoCached)
        {
            self.isVideoCached = NO;
            
            [PokktNetworkDelegate didFinishedAdDownload:self.network adConfig:adConfig rewardValue:0 downLoadTime:@""];
            
            str = @"[Fyber SDK] Fyber video were already saved in cached";
        }
        else
        {
            self.adConfig = adConfig;
            
            FYBRewardedVideoController *rewardedVideoController = [FyberSDK rewardedVideoController];
            rewardedVideoController.delegate = self;
            rewardedVideoController.virtualCurrencyClientDelegate = self;
            FYBRequestParameters *parameters = [[FYBRequestParameters alloc] init];
            [parameters setPlacementId:adConfig.screenName];
            [rewardedVideoController requestVideoWithParameters:parameters];
            
            self.isTimedout = NO;
            
            str = @"[Fyber SDK] Fyber video caching start with timeout timer";
        }
        
        NSLog(@"%@",str);
        
        [self.cacheLock unlock];
    }
    else
    {
        
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (self.currentScreenName == nil)
    {
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
    }
    
    if (![self.currentScreenName isEqualToString:adConfig.screenName])
    {
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
    }
    
    if (!self.isVideoCached)
    {
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
    }
    
    self.playerVC = viewController;
    
    self.isVideoCached = NO;
    self.adConfig = adConfig;
    [[FyberSDK rewardedVideoController] presentRewardedVideoFromViewController:self.playerVC];
}

- (BOOL)checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
        return YES;
    }
    
    return NO;
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;
    
    if (self.currentScreenName == nil)
    {
        flag = NO;
    }
    else if (![self.currentScreenName isEqualToString:adConfig.screenName])
    {
        flag = NO;
    }
    else
    {
        flag = self.isVideoCached;
    }

    [PokktNetworkDelegate onAdAvailabilityStatus:self.network adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"[FyberNetwork SDK] Caching time out");
    self.isTimedout = YES;
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.network;
}

- (void)rewardedVideoControllerDidReceiveVideo:(FYBRewardedVideoController *)rewardedVideoController
{
    NSString *str = nil;
    self.isVideoCached = YES;

    if (self.isTimedout)
    {
        str = @"[Fyber SDK] timeOut";
    }
    else
    {
        str = @"[Fyber SDK] Video downloaded";

        [PokktNetworkDelegate didFinishedAdDownload:self.network adConfig:self.adConfig rewardValue:0 downLoadTime:@""];
    }
    
    NSLog(@"%@",str);
}

- (void)rewardedVideoController:(FYBRewardedVideoController *)rewardedVideoController didFailToReceiveVideoWithError:(NSError *)error
{
    self.currentScreenName = nil;
    self.isVideoCached = NO;
    
    if (self.isTimedout)
    {
        NSLog(@"%@",@"[Fyber SDK] failed to downlaod video after timeOut");
    }
    else
    {
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:self.adConfig errorMessage:error.description];
        NSLog(@"[Fyber SDK] error %@",error.description);
    }
}

#pragma mark FYBRewardedVideoControllerDelegate - Show Video

- (void)rewardedVideoControllerDidStartVideo:(FYBRewardedVideoController *)rewardedVideoController
{
    NSLog(@"[Fyber SDK]Video Started");
    [PokktNetworkDelegate onAdDisplayed:self.network adConfig:self.adConfig];
}

- (void)rewardedVideoController:(FYBRewardedVideoController *)rewardedVideoController didDismissVideoWithReason:(FYBRewardedVideoControllerDismissReason)reason
{
    self.currentScreenName = nil;
    if (reason == FYBRewardedVideoControllerDismissReasonAborted )
    {
        [PokktNetworkDelegate onAdSkipped:self.network adConfig:self.adConfig];
        
        [PokktNetworkDelegate onAdClosed:self.network adConfig:self.adConfig];
        NSLog(@"View Controller Skipped Called");
        
        
    }
    else if (reason == FYBRewardedVideoControllerDismissReasonUserEngaged)
    {
        [PokktNetworkDelegate onAdCompleted:self.network adConfig:self.adConfig];
        
        [PokktNetworkDelegate onAdClosed:self.network adConfig:self.adConfig];
        NSLog(@"View Controller removed Completely");
    }
}

- (void)rewardedVideoController:(FYBRewardedVideoController *)rewardedVideoController didFailToStartVideoWithError:(NSError *)error
{
    self.currentScreenName = nil;
    [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:self.adConfig];
}


#pragma mark - FYBVirtualCurrencyClientDelegate

- (void)virtualCurrencyClient:(FYBVirtualCurrencyClient *)client didReceiveResponse:(FYBVirtualCurrencyResponse *)response
{
    NSLog(@"[Fyber SDK]Received %@ %@", @(response.deltaOfCoins), response.currencyName);
    
    if (self.adConfig.isRewarded)
    {
        float amount = 0.0;
        
        if ([[self.network.customData objectForKey:@"amount"] floatValue])
        {
            amount = [[self.network.customData objectForKey:@"amount"] floatValue];
        }
        
        [PokktNetworkDelegate onAdGratified:self.network adConfig:self.adConfig rewardPoint:amount];
    }
    
}

- (void)virtualCurrencyClient:(FYBVirtualCurrencyClient *)client didFailWithError:(NSError *)error
{
    NSLog(@"[Fyber SDK] Failed to receive virtual currency %@", error.description);
}

@end


@implementation InterstitialFyberNetwork

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.network = customData;
        
    self.cacheLock = [NSRecursiveLock new];
    
    self.isInterstitialCached = NO;
    self.isTimedout = NO;
}

- (void)cacheAd:(PokktAdConfig *)adConfig
{
    if ([self.cacheLock tryLock])
    {
        NSString *str = nil;
        
        if (self.currentScreenName != nil && ![self.currentScreenName isEqualToString:adConfig.screenName])
        {
            [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:@"Caching in Fyber Failed"];
            
            return;
        }
        
        self.currentScreenName = adConfig.screenName;
        
        if (self.isInterstitialCached)
        {
            self.isInterstitialCached = NO;
            
            [PokktNetworkDelegate didFinishedAdDownload:self.network adConfig:adConfig rewardValue:0 downLoadTime:@""];
            
            str = @"[Fyber SDK] Fyber video were already saved in cached";
        }
        else
        {
            self.adConfig = adConfig;
            
            self.isTimedout = NO;
            
            FYBInterstitialController *interstitialController = [FyberSDK interstitialController];
            interstitialController.delegate = self;
            
            FYBRequestParameters *parameters = [[FYBRequestParameters alloc] init];
            [parameters setPlacementId:adConfig.screenName];
            
            [interstitialController requestInterstitialWithParameters:parameters];
            
            
            str = @"[Fyber SDK] Fyber video caching start with timeout timer";
        }
        
        NSLog(@"%@",str);
        
        [self.cacheLock unlock];
    }
    else
    {
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:@"Caching in Fyber Failed"];
    }

}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (self.currentScreenName == nil)
    {
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
    }
    
    if (![self.currentScreenName isEqualToString:adConfig.screenName])
    {
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
    }
    
    if (!self.isInterstitialCached)
    {
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
    }
    
    self.playerVC = viewController;
    
    self.isInterstitialCached = NO;
    self.adConfig = adConfig;
    [[FyberSDK interstitialController] presentInterstitialFromViewController:self.playerVC];
}

- (BOOL)checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        return YES;
    }
    
    return NO;
}


-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;
    
    if (self.currentScreenName == nil)
    {
        flag = NO;
    }
    else if (![self.currentScreenName isEqualToString:adConfig.screenName])
    {
        flag = NO;
    }
    else
    {
        flag = self.isInterstitialCached;
    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.network adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"[FyberNetwork SDK] Caching time out");
    self.isTimedout = YES;
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.network;
}

- (void)interstitialControllerDidReceiveInterstitial:(FYBInterstitialController *)interstitialController
{
    self.isInterstitialCached = YES;
    
    if (self.isTimedout)
    {
        NSLog(@"[Fyber SDK] interstitial downloaded after timeout");
    }
    else
    {
        [PokktNetworkDelegate didFinishedAdDownload:self.network adConfig:self.adConfig rewardValue:0 downLoadTime:@""];
    }
    
}

- (void)interstitialController:(FYBInterstitialController *)interstitialController didFailToReceiveInterstitialWithError:(NSError *)error
{
    self.currentScreenName = nil;
    self.isInterstitialCached = NO;

    if (self.isTimedout)
    {
        NSLog(@"%@",@"[Fyber SDK] failed to downlaod Interstitial after timeOut");
    }
    else
    {
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:self.adConfig errorMessage:error.description];
        NSLog(@"[Fyber SDK] error %@",error.description);
    }
}


#pragma mark - Show Interstitial

- (void)interstitialControllerDidPresentInterstitial:(FYBInterstitialController *)interstitialController
{
    NSLog(@"[Fyber SDK]Interstitial Started");
    [PokktNetworkDelegate onAdDisplayed:self.network adConfig:self.adConfig];
}

- (void)interstitialController:(FYBInterstitialController *)interstitialController didDismissInterstitialWithReason:(FYBInterstitialControllerDismissReason)reason
{
    self.currentScreenName = nil;
    [PokktNetworkDelegate onAdClosed:self.network adConfig:self.adConfig];
    NSLog(@"Interstitial removed Completely");
}

- (void)interstitialController:(FYBInterstitialController *)interstitialController didFailToPresentInterstitialWithError:(NSError *)error
{
    self.currentScreenName = nil;
    [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:self.adConfig];
}

@end

