//
//  HyperMxNetwork.m
//  SampleApp
//
//  Created by Pokkt on 28/11/16.
//  Copyright © 2016 Pokkt. All rights reserved.
//

#import "HyprMXNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"
#import <UIKit/UIKit.h>

#define versionNumber @"2.2"

@implementation HyprMXNetwork

static BOOL isInitialized;

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    if (isInitialized) {
        return;
    }
    isInitialized = YES;
    self.networkModel = customData;
    self.cacheLock  = [NSRecursiveLock new];
    NSString *userId = [NSUUID UUID].UUIDString;
    
    NSString *token = [self.networkModel.customData objectForKey:@"apiToken"];
    if (token != nil && token.length > 0)
    {
        [HYPRMediate initialize: token userId: userId];
        [HYPRMediate setLogLevel:HyprMediateLogLevelDebug];
        [HYPRMediate setDelegate: self];
        [PokktDebugger printLog:@"[HyprMx SDK]HyprMx Initialised"];
    }
    else
    {
        [PokktDebugger printLog:@"[HyprMx SDK] Invalid token... Initialisation failed."];
    }
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if ([self.cacheLock tryLock])
    {
        self.adConfig = adConfig;
        [HYPRMediate checkInventory];
    }
    else
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"HyprMx Ad is not available"];
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (self.isAdAvailable)
    {
         self.adConfig = adConfig;
         [HYPRMediate showAd];
    }
    else
    {
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:self.isAdAvailable];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    self.isTimedout = YES;
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
        return true;
    }
    
    return false;
}

- (void)hyprMediateCanShowAd:(BOOL)adCanBeDisplayed {
    if (adCanBeDisplayed)
    {
        [PokktDebugger printLog:@"HYPRMediate download ad completed"];
        self.isAdAvailable = YES;
        if (!self.isTimedout)
        {
         [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:self.adConfig rewardValue:0.0 downLoadTime:@""];
        }
        else
        {
            [PokktDebugger printLog:@"HyprMx ad download with timeout"];
        }

    }
    else
    {
        [PokktDebugger printLog:@"HYPRMediate does not have an ad to show."];
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:self.adConfig errorMessage:@"HYPRMediate does not have an ad to show"];
    }
}

- (void)hyprMediateRewardDelivered:(HYPRMediateReward *)reward
{
    if (self.adConfig.isRewarded)
    {
        [PokktNetworkDelegate onAdGratified:self.networkModel adConfig:self.adConfig rewardPoint:reward.virtualCurrencyAmount];
    }
}


- (void)hyprMediateErrorOccurred:(HYPRMediateError *)error
{
    [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:self.adConfig errorMessage:error.description];
     self.isAdAvailable = NO;
    [self.cacheLock unlock];
}


- (void)hyprMediateAdStarted
{
    [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:self.adConfig];
}

- (void)hyprMediateAdFinished
{
    [PokktNetworkDelegate onAdCompleted:self.networkModel adConfig:self.adConfig];
    [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:self.adConfig];
    
     self.isAdAvailable = NO;
    [self.cacheLock unlock];
}



@end
