
//  InmobiNetwork.m
//  SampleApp
//
//  Created by Subhendu Sekhar Sahu on 20/10/15.
//  Copyright © 2015 Pokkt. All rights reserved.


#import "InmobiNetwork.h"
#import <UIKit/UIKit.h>
#import "PokktSDK/PokktNetworkDelegate.h"

#define versionNumber @"7.0.0"

@implementation InmobiNetwork

static NSString *networkType = @"InMobi";

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.networkModel = customData;
    
    self.inmobiAdDictionary = [[NSMutableDictionary alloc] initWithCapacity:0];
    
    NSString * accountID = [customData.customData objectForKey:@"AccountId"];
    [IMSdk initWithAccountID:accountID];
    
    if ([PokktDebugger isDebugEnabled])
    {
        [IMSdk setLogLevel:kIMSDKLogLevelDebug];
    }
    
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    InmobiAd * inmobiAd = [self getInmobiAdWithAdConfig:adConfig];
    
    [inmobiAd cacheAd:adConfig];
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    self.viewController = viewController;
    
    InmobiAd * inmobiAd = [self getInmobiAdWithAdConfig:adConfig];
    
    [inmobiAd showAd:adConfig viewController:viewController];
}

- (void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    InmobiAd * inmobiAd = [self getInmobiAdWithAdConfig:adConfig];

    [inmobiAd checkAdAvailability:adConfig];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"Inmobi Caching time out");
    
    if ([self.inmobiAdDictionary objectForKey:adConfig.getKey])
    {
        InmobiAd * inmobiAd = [self.inmobiAdDictionary objectForKey:adConfig.getKey];
        [inmobiAd setCacheTimedOut:adConfig];
    }
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag;
    
    if (adConfig.adFormat == VIDEO || adConfig.adFormat == INTERSTITIAL)
    {
        if ([self.inmobiAdDictionary objectForKey:adConfig.getKey])
        {
            InmobiAd * inmobiAd = [self.inmobiAdDictionary objectForKey:adConfig.getKey];
            flag = [inmobiAd checkAdFormatSupport:adConfig];
        }
        
    }
    
    return flag;
}


- (InmobiAd *)getInmobiAdWithAdConfig: (PokktAdConfig *)adConfig
{
    InmobiAd * inmobiAd = nil;
    
    if ([self.inmobiAdDictionary objectForKey:adConfig.getKey])
    {
        inmobiAd = [self.inmobiAdDictionary objectForKey:adConfig.getKey];
    }
    else
    {
        inmobiAd = [[InmobiAd alloc] init];
        [inmobiAd initNetworkWithNetworkModel:self.networkModel];
        [self.inmobiAdDictionary setObject:inmobiAd forKey:adConfig.getKey];
    }

    return inmobiAd;
}

@end

#pragma mark -
#pragma mark - InmobiAd Class

@implementation InmobiAd

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.networkModel = customData;
    self.adInterstitial = nil;
    self.adConfig = nil;
    self.isTimeout = NO;
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    self.adConfig = adConfig;
    IMInterstitial *adInterstitial = [self getAdInterstitialWithAdConfig:adConfig];
    
    if (adInterstitial != nil)
    {
        if ([adInterstitial isReady])
        {
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
        }
        else
        {
            NSLog(@"Caching started %@", networkType);
            self.isTimeout = NO;
            
            [adInterstitial load];
        }
    }
    else
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"Invalid Screen Name or placement ID"];
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    self.viewController = viewController;
    self.adConfig = adConfig;
    
    if (self.adInterstitial != nil)
    {
        if ([self.adInterstitial isReady])
        {
            [self.adInterstitial showFromViewController:self.viewController];
        }
        else
        {
            [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        }
    }
    else
    {
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }
}

- (void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;
    
    if (self.adInterstitial != nil && [self.adInterstitial isReady])
    {
        flag = YES;
    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"Inmobi Caching time out");
    self.isTimeout = YES;
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag;
    
    if (adConfig.adFormat == VIDEO || adConfig.adFormat == INTERSTITIAL)
    {
        flag = true;
    }
    return flag;
}


#pragma mark - Internal Methods

- (IMInterstitial *)getAdInterstitialWithAdConfig: (PokktAdConfig *)adConfig
{
    if (self.adInterstitial == nil)
    {
        NSDictionary *screenMapdictionary = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
        
        NSString *screen = [screenMapdictionary objectForKey:adConfig.screenName];
        if (screenMapdictionary != nil && screen)
        {
            NSDictionary * networkScreenDetails = [screenMapdictionary objectForKey:adConfig.screenName];
            
            NSString * placementID = [networkScreenDetails objectForKey:@"network_screen"];
            BOOL isRewarded = [[networkScreenDetails objectForKey:@"incent"] boolValue];
            BOOL isNonRewarded = [[networkScreenDetails objectForKey:@"non_incent"] boolValue];
            
            if (placementID != nil && placementID.length !=0)
            {
                if ((isRewarded && adConfig.isRewarded) || !(isNonRewarded && adConfig.isRewarded))
                {
                    self.adInterstitial = [self getAdInterstitialWithPlacementId:placementID andAdConfig:adConfig];
                }
            }
            else
            {
                self.adInterstitial = [self getAdInterstitialWithPlacementId:adConfig.screenName andAdConfig:adConfig];
            }
        }
        else
        {
             self.adInterstitial = [self getAdInterstitialWithPlacementId:adConfig.screenName andAdConfig:adConfig];
        }

    }
    
    return self.adInterstitial;
}


- (IMInterstitial *)getAdInterstitialWithPlacementId: (NSString *)placementId andAdConfig: (PokktAdConfig *)adConfig
{
    long long value = [placementId longLongValue];
    
    IMInterstitial * interstitial = [[IMInterstitial alloc] initWithPlacementId:value delegate:self];
    
    return interstitial;
}

#pragma mark - InMobi Interstitial Delegates

/*Indicates that the interstitial has received an ad. */

- (void)interstitialDidFinishLoading:(IMInterstitial *)interstitial
{
    NSLog(@"[InMobi] interstitialDidFinishLoading");
    
    if (self.isTimeout)
    {
        NSLog(@"[InMobi] cache finished after timeout");
    }
    else
    {
        NSLog(@"[InMobi] cache finished before timeout");

        [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:self.adConfig rewardValue:0 downLoadTime:@""];
    }
}

/* Indicates that the interstitial has failed to receive an ad */
- (void)interstitial:(IMInterstitial *)interstitial didFailToLoadWithError:(IMRequestStatus *)error
{
    
    if (self.isTimeout)
    {
        NSLog(@"[InMobi] cache failed after timeout");
    }
    else
    {
        NSLog(@"[InMobi] cache failed before timeout");
        
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:self.adConfig errorMessage:error.description];
    }
}

/* Indicates that the interstitial has failed to present itself. */
- (void)interstitial:(IMInterstitial *)interstitial didFailToPresentWithError:(IMRequestStatus *)error
{
    NSLog(@"Interstitial didFailToPresentWithError");
    NSLog(@"Error : %@",error.description);
    
    [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:self.adConfig];
}

/* indicates that the interstitial is going to present itself. */
- (void)interstitialWillPresent:(IMInterstitial *)interstitial
{
    NSLog(@"interstitialWillPresent");
}
/* Indicates that the interstitial has presented itself */
- (void)interstitialDidPresent:(IMInterstitial *)interstitial
{
    NSLog(@"interstitialDidPresent");
    
    [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:self.adConfig];
}

/* Indicates that the interstitial is going to dismiss itself. */
- (void)interstitialWillDismiss:(IMInterstitial *)interstitial
{
    NSLog(@"interstitialWillDismiss");
    
    [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:self.adConfig];
}

/* Indicates that the interstitial has dismissed itself. */
- (void)interstitialDidDismiss:(IMInterstitial *)interstitial
{
    NSLog(@"interstitialDidDismiss");
}

/* Indicates that the user will leave the app. */
- (void)userWillLeaveApplicationFromInterstitial:(IMInterstitial *)interstitial
{
    NSLog(@"userWillLeaveApplicationFromInterstitial");
}

/* Indicates that a reward action is completed */
- (void)interstitial:(IMInterstitial *)interstitial rewardActionCompletedWithRewards:(NSDictionary *)rewards
{
    // Extract the reward points as  key entered in the inmobi dashbpard
    
    NSLog(@"IncentActionCompleted Publisher Callback successfully received");
    [PokktNetworkDelegate onAdCompleted:self.networkModel adConfig:self.adConfig];
    if (self.adConfig.isRewarded && [rewards objectForKey:@"reward"] != nil && [[rewards objectForKey:@"reward"] length] > 0)
    {
        [PokktNetworkDelegate onAdGratified:self.networkModel adConfig:self.adConfig rewardPoint:[[rewards objectForKey:@"reward"] floatValue]];
    }

}

/* interstitial:didInteractWithParams: Indicates that the interstitial was interacted with. */
- (void)interstitial:(IMInterstitial *)interstitial didInteractWithParams:(NSDictionary *)params
{
    NSLog(@"InterstitialDidInteractWithParams");
}

@end
