//
//  IronSourceNetwork.m
//  PokktAdsDemo
//
//  Created by Pokkt on 02/03/17.
//  Copyright © 2017 Pokkt. All rights reserved.
//

#import "IronSourceNetwork.h"

#import "PokktSDK/PokktNetworkDelegate.h"



#define versionNumber @"6.6.7.1"

@implementation IronSourceNetwork

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.networkModel = customData;
    
    NSString *appKey = [customData.customData objectForKey:@"appKey"];//@"5fa96f65"
    if (appKey != nil && appKey.length > 0)
    {
        
        [ISIntegrationHelper validateIntegration];
        [ISSupersonicAdsConfiguration configurations].useClientSideCallbacks = @(YES);
        
        //Supersonic tracking SDK
        [ISEventsReporting reportAppStarted];
        
        
        NSString *userId = [IronSource advertiserId];
        
        if([userId length] == 0){
            //If we couldn't get the advertiser id, we will use a default one.
            userId = @"demoapp";
        }
        
        // After setting the delegates you can go ahead and initialize the SDK.
        [IronSource setUserId:userId];
        [IronSource initWithAppKey:appKey];
        
    }

    
    self.videoAd = [[IronSourceVideoAds alloc] init];
    [self.videoAd initNetworkWithNetworkModel:customData];
    
    self.interstitialAd = [[IronSourceInterstitialAds alloc] init];
    [self.interstitialAd initNetworkWithNetworkModel:customData];
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
        [self.videoAd cacheAd:adConfig];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.interstitialAd cacheAd:adConfig];
    }
    
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (adConfig.adFormat == VIDEO)
    {
        [self.videoAd showAd:adConfig viewController:viewController];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
       [self.interstitialAd showAd:adConfig viewController:viewController];
    }
    
}


-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
         [self.videoAd checkAdAvailability:adConfig];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
         [self.interstitialAd checkAdAvailability:adConfig];
    }
   
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
        [self.videoAd setCacheTimedOut:adConfig];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.interstitialAd setCacheTimedOut:adConfig];
    }
    
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == VIDEO)
    {
        flag = [self.videoAd checkAdFormatSupport:adConfig];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        flag = [self.interstitialAd checkAdFormatSupport:adConfig];
    }
    
    return flag;
}

@end


@implementation IronSourceVideoAds

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
     self.networkModel = customData;
    
    [IronSource setRewardedVideoDelegate:self];
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    self.adConfig = adConfig;
    if ([IronSource hasRewardedVideo])
    {
        [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
    }
    else
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"Mobile core Ad is not available"];
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    self.adConfig = adConfig;
    if ([IronSource hasRewardedVideo])
    {
       [IronSource showRewardedVideoWithViewController:viewController placement:@"RewardVideoAd"];
    }
    else
    {
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }
}


-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:self.isVideoAvailable];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    self.isTimeOut = YES;
}



-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == VIDEO)
    {
        flag = YES;
    }
    
    return flag;
}

#pragma mark Video ads delegates

- (void)rewardedVideoHasChangedAvailability:(BOOL)available;
{
    self.isVideoAvailable = available;
    
    if (self.isTimeOut)
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:self.adConfig errorMessage:@"Timeout error"];
    }
      
}


- (void)didReceiveRewardForPlacement:(ISPlacementInfo *)placementInfo
{
    if (placementInfo.rewardAmount.intValue > 0)
    {
        [PokktNetworkDelegate onAdGratified:self.networkModel adConfig:self.adConfig rewardPoint:placementInfo.rewardAmount.floatValue];
    }
}


- (void)rewardedVideoDidFailToShowWithError:(NSError *)error
{
  [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:self.adConfig];
}


- (void)rewardedVideoDidOpen
{
    [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:self.adConfig];
}

- (void)rewardedVideoDidClose
{
    [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:self.adConfig];
}

- (void)rewardedVideoDidStart
{
    
}

- (void)rewardedVideoDidEnd
{
    [PokktNetworkDelegate onAdCompleted:self.networkModel adConfig:self.adConfig];
}

@end

@implementation IronSourceInterstitialAds

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.networkModel = customData;
    [IronSource setInterstitialDelegate:self];
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if (adConfig.isRewarded)
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"Invalid network"];
        return;
    }
    
    self.adConfig = adConfig;
    if ([IronSource hasInterstitial])
    {
        [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
    }
    else
    {
        [IronSource loadInterstitial];
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (adConfig.isRewarded)
    {
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        return;
    }
    
    self.adConfig = adConfig;
    if ([IronSource hasInterstitial])
    {
        [IronSource showInterstitialWithViewController:viewController placement:@"testInt"];
    }
    else
    {
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }
}


-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:[IronSource hasInterstitial]];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    self.isTimeOut = YES;
}



-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == INTERSTITIAL)
    {
        flag = YES;
    }
    
    return flag;
}

- (void)interstitialDidLoad
{
    if (!self.isTimeOut)
    {
        [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:self.adConfig rewardValue:0 downLoadTime:@""];
    }
    else
    {
        NSLog(@"IronSource cahing failed with in timeout");
    }
}


- (void)interstitialDidFailToLoadWithError:(NSError *)error
{
    if (self.isTimeOut)
    {
        [PokktDebugger printLog:@"[IronSource SDK] Ad download failed with timeout"];
    }
    else
    {
        NSString *errorMsg = [NSString stringWithFormat:@"[IronSource SDK] Ad download failed before timeout with error %@",error.description];
        [PokktDebugger printLog:errorMsg];
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:self.adConfig errorMessage:errorMsg];
    }
}

- (void)interstitialDidOpen
{
    [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:self.adConfig];
}

- (void)interstitialDidClose
{
    [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:self.adConfig];
}

- (void)interstitialDidShow
{
    
}

- (void)interstitialDidFailToShowWithError:(NSError *)error
{
    [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:self.adConfig];
}

- (void)didClickInterstitial
{
    
}





@end
