//
//  LeadBoltNetwork.m
//  PokktAdsDemo
//
//  Created by Pokkt on 08/03/17.
//  Copyright © 2017 Pokkt. All rights reserved.
//

#import "LeadBoltNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"

#define versionNumber @"8.1"

@implementation LeadBoltNetwork

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.cacheLock  = [NSRecursiveLock new];
    self.networkModel = customData;
    
    NSString *appId = [customData.customData objectForKey:@"appId"];//@"8uc8Kd5b6LoyJZCAsFUCY2sWmSJkXZ6c"
    if (appId != nil && appId.length > 0)
    {
        [AppTracker setAppModuleDelegate:self];
        [AppTracker startSession:appId withOption:AppTrackerEnableAutoCache];
    }

}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if ([self.cacheLock tryLock])
    {
        NSString *placementId = [self getPlacementIdwithAdConfig:adConfig];
        
        if (placementId == nil || placementId.length == 0) {
            [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"Invalid Screenname.."];
            return;
        }
        
        if ([AppTracker isAdReady:placementId])
        {
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
            return;
        }
        else
        {
             [AppTracker loadModuleToCache:placementId];
        }
    }
    else
    {
         [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"LeadBolt Ad is not available"];
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    NSString *placementId = [self getPlacementIdwithAdConfig:adConfig];
    
    if (placementId == nil || placementId.length == 0) {
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        return;
    }
    if ([AppTracker isAdReady:placementId])
    {
         [AppTracker loadModule:placementId viewController:viewController];
    }
    else
    {
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    NSString *placementId = [self getPlacementIdwithAdConfig:adConfig];
    
    if (placementId == nil || placementId.length == 0) {
        [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:flag];
        return;
    }
   
    if ([AppTracker isAdReady:placementId])
    {
        flag = true;
    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    self.isTimeOut = YES;
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == INTERSTITIAL)
    {
        flag = true;
    }
    
    return flag;
}

-(NSString *)getPlacementIdwithAdConfig:(PokktAdConfig *)adConfig
{
    NSString *placementID;
    
    NSDictionary *screenMapdictionary = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
    
    if (screenMapdictionary != nil && [screenMapdictionary objectForKey:adConfig.screenName])
    {
        NSDictionary * networkScreenDetails = [screenMapdictionary objectForKey:adConfig.screenName];
        placementID = [networkScreenDetails objectForKey:@"network_screen"];
    }
    
    return placementID;
}

-(void) onModuleLoaded:(NSString *)placement
{
    
}

-(void) onModuleClosed:(NSString *)placement
{
    [self.cacheLock unlock];
    [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:self.adConfig];
}

-(void) onModuleClicked:(NSString *)placement
{
    
}

-(void) onModuleCached:(NSString *)placement
{
    if (!self.isTimeOut) {
        [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:self.adConfig rewardValue:0 downLoadTime:@""];
    }
 
}

-(void) onModuleFailed:(NSString *)placement error:(NSString *)error cached:(BOOL)iscached
{
    [self.cacheLock unlock];
    if (!self.isTimeOut) {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:self.adConfig errorMessage:error.description];
    }
}

-(void) onMediaFinished:(BOOL)viewCompleted
{
    [PokktNetworkDelegate onAdCompleted:self.networkModel adConfig:self.adConfig];
}


@end
