//
//  LoopMeNetwork.m
//  SampleApp
//
//  Created by Pokkt on 03/11/16.
//  Copyright © 2016 Pokkt. All rights reserved.
//

#import "LoopMeNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"

#define versionNumber @"6.4.1"

static NSString *networkType = @"LoopMe";
static BOOL isInitialized;
@implementation LoopMeNetwork

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    if (isInitialized) {
        return;
    }
    isInitialized = YES;
    self.networkModel = customData;
    
    self.interstitialAd = [[InterstitialAds alloc] init];
    [self.interstitialAd initNetworkWithNetworkModel:customData];
    
    self.bannerAd = [[LoopMeBanner alloc] init];
    self.bannerAd.networkModel = customData;

}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    [self.interstitialAd cacheAd:adConfig];
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    [self.interstitialAd showAd:adConfig viewController:viewController];
}

-(void) fetchAd:(PokktAdConfig *)adConfig withAdView:(UIView *)bannerView withRootViewController:(UIViewController *)rootViewController withDelegate:(id<PokktBannerDelegate>)bannerDelegate onSuccess:(void (^)(id))successCallback onFailure:(void (^)(id))failureHandler
{
    [self.bannerAd fetchAd:adConfig withAdView:bannerView withRootViewController:rootViewController withDelegate:bannerDelegate onSuccess:successCallback onFailure:failureHandler];
}



-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    [self.interstitialAd checkAdAvailability:adConfig];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"[LoopMe SDK] Caching time out");
    [self.interstitialAd setCacheTimedOut:adConfig];
    
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}


- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == BANNER)
    {
       return [self.bannerAd checkAdFormatSupport:adConfig];
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        return [self.interstitialAd checkAdFormatSupport:adConfig];
    }
    
    return false;
}

@end

@implementation LoopMeBanner

-(void) fetchAd:(PokktAdConfig *)adConfig withAdView:(UIView *)bannerView withRootViewController:(UIViewController *)rootViewController withDelegate:(id<PokktBannerDelegate>)bannerDelegate onSuccess:(void (^)(id))successCallback onFailure:(void (^)(id))failureHandler
{
    self.bannerDelegate = bannerDelegate;
    self.rootViewController = rootViewController;
    NSString *screenName = [self.networkModel.customData objectForKey:@"AppKey"];
    if (screenName != nil && screenName.length != 0)
    {
        
        CGRect adFrame = CGRectMake(0, 0, self.bannerParentView.frame.size.width , self.bannerParentView.frame.size.height);
        LoopMeAdView *adView   =  [LoopMeAdView adViewWithAppKey:screenName
                                 frame:adFrame
                              delegate:self];
        [adView loadAd];
        
        [bannerView addSubview:adView];
        
    }
    else
    {
       [self.bannerDelegate bannerLoadFailed:adConfig.screenName errorMessage:@"[LoopMe SDK] Invalid adConfig"];
    }
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == BANNER)
    {
        return true;
    }
    return false;
}

- (UIViewController *)viewControllerForPresentation
{
    return self.rootViewController;
}

- (void)loopMeAdViewDidLoadAd:(LoopMeAdView *)adView
{
     [self.bannerDelegate bannerLoaded:adView.appKey];
}

- (void)loopMeAdView:(LoopMeAdView *)adView didFailToLoadAdWithError:(NSError *)error
{
    dispatch_sync(dispatch_get_main_queue(), ^{
        [adView removeFromSuperview];
        [self.bannerDelegate bannerLoadFailed:adView.appKey errorMessage:error.description];
        
    });
    
}

- (void)loopMeAdViewVideoDidReachEnd:(LoopMeAdView *)adView
{
    
}

- (void)loopMeAdViewDidExpire:(LoopMeAdView *)adView;
{
    
}

- (void)loopMeAdViewDidReceiveTap:(LoopMeAdView *)adView;
{
    
}

- (void)loopMeAdViewWillLeaveApplication:(LoopMeAdView *)adView
{
    
}

@end

@implementation InterstitialAds

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    NSLog(@"%@ is initialised", networkType);
    [PokktDebugger printLog:@"Initialising LoopMe SDK"];
    
    self.networkModel  =customData;
    self.isFullyWatched = NO;
    self.cacheLock  = [NSRecursiveLock new];
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
        [PokktDebugger printLog:@"[LoopMe SDK] Invalid network for caching ad"];
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"[LoopMe SDK] Invalid network"];
        
        return;
    }
    NSLog(@"%@ video caching started", networkType);

    self.adConfig = adConfig;
    if ([self.cacheLock tryLock])
    {
       NSString *screenName = [self.networkModel.customData objectForKey:@"AppKey"];
        if (screenName != nil && screenName.length > 0)
        {
            self.loopMeAd = [LoopMeInterstitial interstitialWithAppKey:adConfig.screenName delegate:self];
            if ([self.loopMeAd isReady])
            {
                [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0.0 downLoadTime:@""];
            }
            else
            {
                
                [self.loopMeAd loadAd];                
            }
        }
        else
        {
            [PokktDebugger printLog:@"[LoopMe SDK] Invalid screenname"];
            [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"[LoopMe SDK] Invalid screenname"];
        }
    }
    else
    {
        [PokktDebugger printLog:@"[LoopMe SDK] Caching is already in progress..."];
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"[LoopMe SDK] Caching is already in progress..."];
    }
 }

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (adConfig.adFormat == VIDEO)
    {
        [PokktDebugger printLog:@"[LoopMe SDK] Invalid network for show ad"];
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        return;
    }
    
    if (self.loopMeAd != nil && self.loopMeAd.isReady)
    {
        self.adConfig = adConfig;
        [self.loopMeAd showFromViewController:viewController animated:YES];
    }
    else
    {
        [PokktDebugger printLog:@"[LoopMe SDK] Failed to show ad"];
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }

}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
        [PokktDebugger printLog:@"[LoopMe SDK] Invalid network for check ad availability"];
         [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:false];
        return;
    }
    BOOL flag = NO;
    if ([self.loopMeAd isReady])
    {
        flag = YES;
    }
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"[LoopMe SDK] Caching time out");
    self.isTimedout = YES;
    
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        return true;
    }
    
    return false;
}

- (void)loopMeInterstitial:(LoopMeInterstitial *)interstitial didFailToLoadAdWithError:(NSError *)error
{
   
    if (self.isTimedout)
    {
        [PokktDebugger printLog:@"[LoopMe SDK] Ad download failed with timeout"];
    }
    else
    {
        NSString *errorMsg = [NSString stringWithFormat:@"[LoopMe SDK] Ad download failed before timeout with error %@",error.description];
        [PokktDebugger printLog:errorMsg];
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:self.adConfig errorMessage:errorMsg];
    }
    
     [self.cacheLock unlock];
    
}

- (void)loopMeInterstitialDidLoadAd:(LoopMeInterstitial *)interstitial
{
    if (self.isTimedout)
    {
        [PokktDebugger printLog:@"[LoopMe SDK] Ad download successfully with timeout"];
    }
    else
    {
        [PokktDebugger printLog:@"[LoopMe SDK] Ad download successfully before timeout"];
        [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:self.adConfig rewardValue:0.0 downLoadTime:@""];
    }
}

- (void)loopMeInterstitialWillAppear:(LoopMeInterstitial *)interstitial
{
    [PokktDebugger printLog:@"[LoopMe SDK] Ad Displayed"];
    [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:self.adConfig];
}

- (void)loopMeInterstitialVideoDidReachEnd:(LoopMeInterstitial *)interstitial
{
    [PokktDebugger printLog:@"[LoopMe SDK] Ad complete watched"];
    [PokktNetworkDelegate onAdCompleted:self.networkModel adConfig:self.adConfig];
}

- (void)loopMeInterstitialDidExpire:(LoopMeInterstitial *)interstitial
{
    [PokktDebugger printLog:@"[LoopMe SDK] Ad expire"];
    [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:self.adConfig];
    [self.cacheLock unlock];
}

- (void)loopMeInterstitialDidDisappear:(LoopMeInterstitial *)interstitial
{
    [PokktDebugger printLog:@"[LoopMe SDK] Ad disapper"];
    
    [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:self.adConfig];
    
    [self.cacheLock unlock];
}

@end
