//
//  MillennialNetwork.m
//  PokktAdsDemo
//
//  Created by Pokkt on 27/02/17.
//  Copyright © 2017 Pokkt. All rights reserved.
//

#import "MillennialNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"
#import <MMAdSDK/MMAdSDK.h>

#define versionNumber @"6.4.0"


@implementation MillennialNetwork


- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.networkModel = customData;
    
    self.interstitialAd = [[MillennialInterstitialAds alloc] init];
    [self.interstitialAd initNetworkWithNetworkModel:customData];
    
    self.bannerAd = [[MillennialBanner alloc] init];
    self.bannerAd.networkModel = customData;

}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    [self.interstitialAd cacheAd:adConfig];
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    [self.interstitialAd showAd:adConfig viewController:viewController];
}

-(void) fetchAd:(PokktAdConfig *)adConfig withAdView:(UIView *)bannerView withRootViewController:(UIViewController *)rootViewController withDelegate:(id<PokktBannerDelegate>)bannerDelegate onSuccess:(void (^)(id))successCallback onFailure:(void (^)(id))failureHandler
{
    [self.bannerAd fetchAd:adConfig withAdView:bannerView withRootViewController:rootViewController withDelegate:bannerDelegate onSuccess:successCallback onFailure:failureHandler];
}


-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    [self.interstitialAd checkAdAvailability:adConfig];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    [self.interstitialAd setCacheTimedOut:adConfig];
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == INTERSTITIAL)
    {
      flag = [self.interstitialAd checkAdFormatSupport:adConfig];
    }
    else if (adConfig.adFormat == BANNER)
    {
        flag = [self.bannerAd checkAdFormatSupport:adConfig];
    }
    
    return flag;
}


@end

#pragma mark Interstitial Network

@implementation MillennialInterstitialAds

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    NSLog(@"MillennialNetwork is initialised");
    
    self.placementDictionary = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.adConfigDictionary = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.timeoutDictionary = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.networkModel = customData;
    [MMSDK setLogLevel:MMLogLevelDebug];
    [[MMSDK sharedInstance] initializeWithSettings:nil withUserSettings:nil];
    
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    
    if (adConfig.isRewarded)
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"MillennialNetwork ad is not available"];
        return;
    }
    
    MMInterstitialAd *placement = (MMInterstitialAd *)[self getPlacementwithAdConfig:adConfig];
    
    if (placement == nil)
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"MillennialNetwork placement content is not available"];
        return;
    }
    
    [self.placementDictionary setObject:placement forKey:adConfig.getKey];
    [self.adConfigDictionary setObject:adConfig forKey:placement.placementId];
    
    if (placement.ready)
    {
        NSLog(@"MillennialNetwork cached previously");
        
        [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
    }
    else
    {
        NSLog(@"MillennialNetwork caching started");
        
        [self.timeoutDictionary setObject:[NSNumber numberWithBool:NO] forKey:adConfig.getKey];
        placement.delegate = self;
        [placement load:nil];
        
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if(adConfig.isRewarded)
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"MillennialNetwork Invalid Network"];
        return;
    }
    
    MMInterstitialAd *placement = [self getPlacementwithAdConfig:adConfig];
    
    if (placement != nil && placement.ready)
    {
        [placement showFromViewController:viewController];
    }
    else
    {
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;
    
    MMInterstitialAd *placement = [self getPlacementwithAdConfig:adConfig];
    
    if (placement != nil && placement.ready)
    {
        flag = YES;
    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"MillennialNetwork Caching time out");
    [self.timeoutDictionary setObject:[NSNumber numberWithBool:YES] forKey:adConfig.getKey];
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == INTERSTITIAL)
    {
        flag = true;
    }
    
    return flag;
}

-(MMInterstitialAd *)getPlacementwithAdConfig:(PokktAdConfig *)adConfig
{
    MMInterstitialAd *placement;
    
    if ([self.placementDictionary objectForKey:adConfig.getKey])
    {
        placement = [self.placementDictionary objectForKey:adConfig.getKey];
    }
    else
    {
        NSDictionary *screenMapdictionary = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
        
        if (screenMapdictionary != nil && [screenMapdictionary objectForKey:adConfig.screenName])
        {
            NSDictionary * networkScreenDetails = [screenMapdictionary objectForKey:adConfig.screenName];
            
            NSString * placementName = [networkScreenDetails objectForKey:@"network_screen"];
            
            if (placementName != nil && placementName.length !=0)
            {
                   placement = [[MMInterstitialAd alloc] initWithPlacementId:placementName];
            }
        }
    }
    
    return placement;
 
}

#pragma mark - Interstitial Delegate

- (void)interstitialAdLoadDidSucceed:(MMInterstitialAd *)ad {
    NSLog(@"MillennialNetwork Interstitial did load.");

    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:ad.placementId];
    
    if (adConfig != nil)
    {
        if ([[self.timeoutDictionary objectForKey:adConfig.getKey] boolValue])
        {
            NSLog(@"MillennialNetwork caching failed after timeout");
        }
        else
        {
            NSLog(@"MillennialNetwork download completed");
            
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
        }
    }
    else
    {
        NSLog(@"Unknown placement");
    }
}

- (void)interstitialAd:(MMInterstitialAd *)ad loadDidFailWithError:(NSError *)error
{
    NSLog(@"MillennialNetwork Interstitial load failed: %@.", error);
    
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:ad.placementId];
    
    if (adConfig)
    {
        if ([[self.timeoutDictionary objectForKey:adConfig.getKey] boolValue])
        {
            NSLog(@"MillennialNetwork caching failed after timeout");
        }
        else
        {
            NSLog(@"MillennialNetwork download failed");
            
            [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:error.description];
        }
    }
    
}

- (void)interstitialAdWillDisplay:(MMInterstitialAd *)ad {
    NSLog(@"Interstitial will display.");
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:ad.placementId];
    
    if (adConfig)
    {
        [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:adConfig];
    }
    else
    {
        NSLog(@"Unknown placement");
    }

}

- (void)interstitialAdDidDisplay:(MMInterstitialAd *)ad {
    NSLog(@"Interstitial did display.");
  
    
}

- (void)interstitialAdWillDismiss:(MMInterstitialAd *)ad {
    NSLog(@"Interstitial will dismiss.");
    
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:ad.placementId];
    
    if (adConfig)
    {
        [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:adConfig];
    }
    else
    {
        NSLog(@"Unknown placement");
    }

}

- (void)interstitialAdDidDismiss:(MMInterstitialAd *)ad {
    NSLog(@"Interstitial did dismiss.");
    
}

- (void)interstitialAdTapped:(MMInterstitialAd *)ad {
    NSLog(@"Interstitial tapped.");
}

- (void)interstitialAdWillLeaveApplication:(MMInterstitialAd *)ad {
    NSLog(@"Interstitial ad will leave application.");
}

@end

@implementation MillennialBanner

-(void) fetchAd:(PokktAdConfig *)adConfig withAdView:(UIView *)bannerView withRootViewController:(UIViewController *)rootViewController withDelegate:(id<PokktBannerDelegate>)bannerDelegate onSuccess:(void (^)(id))successCallback onFailure:(void (^)(id))failureHandler
{
    NSLog(@"Aerserv banner fetch request");
    self.bannerDelegate = bannerDelegate;
    self.rootViewController = rootViewController;
    NSString *placementId = [self getPlacementIdwithAdConfig:adConfig];
    if (placementId != nil && placementId.length != 0)
    {
        MMInlineAd *inlineAd = [[MMInlineAd alloc] initWithPlacementId:placementId
                                                         size:CGSizeMake( bannerView.frame.size.width, bannerView.frame.size.height)];
        inlineAd.delegate = self;
        [bannerView addSubview:inlineAd.view];
        [inlineAd request:nil];
    }
    else
    {
        [self.bannerDelegate bannerLoadFailed:adConfig.screenName errorMessage:@"[AerServ SDK] Invalid adConfig"];
    }
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == BANNER)
    {
        return true;
    }
    return false;
}

- (UIViewController *)viewControllerForPresentingModalView
{
    return self.rootViewController;
}

-(NSString *)getPlacementIdwithAdConfig:(PokktAdConfig *)adConfig
{
    NSString *placementID;
    
    NSDictionary *screenMapdictionary = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
    
    if (screenMapdictionary != nil && [screenMapdictionary objectForKey:adConfig.screenName])
    {
        NSDictionary * networkScreenDetails = [screenMapdictionary objectForKey:adConfig.screenName];
        placementID = [networkScreenDetails objectForKey:@"network_screen"];
    }
    
    return placementID;
}

-(void)inlineAdRequestDidSucceed:(MMInlineAd*)ad
{
    [self.bannerDelegate bannerLoaded:@""];
}

-(void)inlineAd:(MMInlineAd*)ad requestDidFailWithError:(NSError*)error
{
    [self.bannerDelegate bannerLoadFailed:@"" errorMessage:error.description];
}


-(void)inlineAdContentTapped:(MMInlineAd*)ad
{
    
}

-(void)inlineAd:(MMInlineAd*)ad willResizeTo:(CGRect)frame isClosing:(BOOL)isClosingResize
{
    
}

-(void)inlineAd:(MMInlineAd*)ad didResizeTo:(CGRect)frame isClosing:(BOOL)isClosingResize
{
    
}


-(void)inlineAdWillPresentModal:(MMInlineAd *)ad
{
    
}


-(void)inlineAdDidPresentModal:(MMInlineAd *)ad
{
    
}


-(void)inlineAdWillCloseModal:(MMInlineAd *)ad
{
    
}

-(void)inlineAdDidCloseModal:(MMInlineAd *)ad
{
    
}


-(void)inlineAdWillLeaveApplication:(MMInlineAd *)ad
{
    
}


-(void)inlineAdAbortDidSucceed:(MMInlineAd*)ad
{
    
}


-(void)inlineAd:(MMInlineAd*)ad abortDidFailWithError:(NSError*)error
{
    
}


@end
