//
//  NativeXNetwork.m
//  SampleApp
//
//  Created by Pokkt on 25/11/16.
//  Copyright © 2016 Pokkt. All rights reserved.
//

#import "NativeXNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"

#define versionNumber @"5.5.9"

@implementation NativeXNetwork

static BOOL isInitialized;

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    if (isInitialized) {
        return;
    }
    isInitialized = YES;
    self.networkModel = customData;
    
    NSLog(@"[NativeX SDK]NativeX Initialised");
    
    self.adConfigDict = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.timeOutDict = [[NSMutableDictionary alloc] initWithCapacity:0];
    
    //20910 samplePubUserId
    NSString *appId = [self.networkModel.customData objectForKey:@"appId"];
    if (appId != nil && appId.length > 0)
    {
        [NativeXSDK setSDKDelegate:self];
        [NativeXSDK enableDebugLog:true];

        [NativeXSDK initializeWithAppId:appId];
    }
    else
    {
        [PokktDebugger printLog:@"[NativeX SDK] Invalid AppId.... Initialisation Failed."];
    }
    
    
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    NSString *screenName = [self getScreenName:adConfig];
    if (screenName != nil && screenName.length > 0)
    {
        if (![NativeXSDK isAdFetchedWithName:screenName])
        {
            [self.adConfigDict setValue:adConfig forKey:screenName];
            [NativeXSDK setAdEventDelegate:self];
            [NativeXSDK setRewardDelegate:self];
            [NativeXSDK fetchAdsAutomaticallyWithName:screenName];
        }
        else
        {
            NativeXAdInfo *info = [NativeXSDK getAdInfoWithName:screenName];
            
            if ((adConfig.adFormat == VIDEO && info.willPlayAudio) || (adConfig.adFormat == INTERSTITIAL))
            {
               [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0.0 downLoadTime:@""];
            }
            else
            {
              [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"[NativeX SDK] Invalid adFormat"];
            }
        }
    }
    else
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"[NativeX SDK] Invalid screenname"];
    }

}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    NSString *screenName = [self getScreenName:adConfig];
    if (screenName != nil && screenName.length > 0)
    {
        if ([NativeXSDK isAdFetchedWithName:screenName])
        {
            NativeXAdInfo *info = [NativeXSDK getAdInfoWithName:screenName];
            
            if ((adConfig.adFormat == VIDEO && info.willPlayAudio) || (adConfig.adFormat == INTERSTITIAL))
            {
                if (info.adState != AdStateExpired)
                {
                    [self.adConfigDict setValue:adConfig forKey:screenName];
                    [NativeXSDK showAdWithName:screenName andShowDelegate:self rootViewController:viewController];
                }
                else
                {
                    [PokktDebugger printLog:@"[NativeX SDK] Ad has expired.."];
                   [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
                }
            }
            else
            {
              [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];  
            }
            
        }
        else
        {
          [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        }
    }
    else
    {
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }
    
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;
    NSString *screenName = [self getScreenName:adConfig];
    if (screenName != nil && screenName.length > 0)
    {
        NativeXAdInfo *info = [NativeXSDK getAdInfoWithName:screenName];
        
        if (info != nil)
        {
            if ((adConfig.adFormat == VIDEO && info.willPlayAudio) || (adConfig.adFormat == INTERSTITIAL))
            {
                if ([NativeXSDK isAdFetchedWithName:screenName])
                {
                    flag = YES;
                }
            }

        }
        
    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSString *screenName = [self getScreenName:adConfig];
    if (screenName != nil && screenName.length > 0)
    {
        [self.timeOutDict setValue:[NSNumber numberWithBool:YES] forKey:screenName];
    }
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == VIDEO)
    {
        return true;
    }
    else if (adConfig.adFormat == INTERSTITIAL)
    {
        return true;
    }
    
    return false;
}

-(NSString *)getScreenName:(PokktAdConfig *)adConfig
{
    NSString *screenName = @"";
    
    if ([self.networkModel.customData objectForKey:@"screens_mapping_data"] != nil && [self.networkModel.customData objectForKey:@"screens_mapping_data"])
    {
        NSDictionary *screenDict = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
        
        if ([screenDict objectForKey:adConfig.screenName] != nil && [screenDict objectForKey:adConfig.screenName])
        {
            NSDictionary *networkDict = [screenDict objectForKey:adConfig.screenName];
            
            if ([networkDict objectForKey:@"network_screen"] != nil && [networkDict objectForKey:@"network_screen"])
            {
                screenName = [networkDict objectForKey:@"network_screen"];
            }
        }
    }
    else
    {
        screenName = adConfig.screenName;
    }
    
    return screenName;
}

- (void)nativeXSDKInitialized
{
    
}

- (void)nativeXSDKFailedToInitializeWithError:(NSError*) error
{
    
}

- (void) adFetched:(NSString*)placementName
{
    BOOL timeOut = [[self.timeOutDict objectForKey:placementName] boolValue];

    if (!timeOut)
    {
        PokktAdConfig *adConfig = [self.adConfigDict objectForKey:placementName];
        
        NativeXAdInfo *info = [NativeXSDK getAdInfoWithName:placementName];
        
        if (info != nil)
        {
            if ((adConfig.adFormat == VIDEO && info.willPlayAudio) || (adConfig.adFormat == INTERSTITIAL))
            {
              [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0.0 downLoadTime:@""];
            }
        }
    }
    else
    {
        [PokktDebugger printLog:@"[Native SDK] ad download with timeout"];
    }
}

- (void) noAdAvailable:(NSString*)placementName
{
    BOOL timeOut = [[self.timeOutDict objectForKey:placementName] boolValue];
    if (!timeOut)
    {
        PokktAdConfig *adConfig = [self.adConfigDict objectForKey:placementName];
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"[Nativex SDK]No ad available."];
    }
  
}

- (void) adShown:(NSString*)placementName
{
    PokktAdConfig *adConfig = [self.adConfigDict objectForKey:placementName];
    [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:adConfig];
}

- (void) adDismissed:(NSString*)placementName converted:(BOOL)converted
{
    PokktAdConfig *adConfig = [self.adConfigDict objectForKey:placementName];
    self.adConfig = adConfig;
    [PokktNetworkDelegate onAdCompleted:self.networkModel adConfig:adConfig];
    [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:adConfig];
}

- (void) adFailedToShow:(NSString*)placementName withError:(NSError*)error
{
    PokktAdConfig *adConfig = [self.adConfigDict objectForKey:placementName];

    [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
}

- (void) adFetchFailed:(NSString*)placementName withError:(NSError*)error
{
        BOOL timeOut = [[self.timeOutDict objectForKey:placementName] boolValue];
        if (!timeOut)
        {
            PokktAdConfig *adConfig = [self.adConfigDict objectForKey:placementName];
            [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:error.description];
        }
        else
        {
            [PokktDebugger printLog:@"[Native SDK] ad download failed with timeout"];
        }
  
}

- (void) rewardAvailable:(NativeXRewardInfo*) rewardInfo
{
    for( NativeXReward *rewardDetail in rewardInfo.rewards)
    {
        if (self.adConfig.isRewarded)
        {
            [PokktNetworkDelegate onAdGratified:self.networkModel adConfig:self.adConfig rewardPoint:rewardDetail.amount.floatValue];
        }
        
    }
}

@end
