//
//  RevMobNetwork.m
//  PokktAdsDemo
//
//  Created by Pokkt on 03/03/17.
//  Copyright © 2017 Pokkt. All rights reserved.
//

#import "RevMobNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"



#define versionNumber @"10.0.0"

@implementation RevMobNetwork

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.networkModel = customData;
    
    
  
    self.videoAdConfigDict = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.videoTimeOutDict = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.intertitialAdConfigDict = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.intertitialTimeOutDict = [[NSMutableDictionary alloc] initWithCapacity:0];

    NSString *appId = [customData.customData objectForKey:@"appId"];
    if (appId != nil && appId.length > 0)
    {
        [Revmob initWithAppId:appId];
        [Revmob setDelegate:self];
    }
    
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        NSString *placementId = [self getAdPlacementId:adConfig];
        if ([Revmob hasAdCachedOnAdUnit:RMInterstitial withPlacement:placementId])
        {
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
        }
        else
        {
            [Revmob cacheInterstitial:placementId];
        }
    }
    else if (adConfig.adFormat == VIDEO)
    {
        NSString *placementId = [self getAdPlacementId:adConfig];
        if ([Revmob hasAdCachedOnAdUnit:RMRewardedVideo withPlacement:placementId])
        {
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
        }
        else
        {
            [Revmob cacheRewardedVideo:placementId];
        }
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        NSString *placementId = [self getAdPlacementId:adConfig];
        if ([Revmob hasAdCachedOnAdUnit:RMInterstitial withPlacement:placementId])
        {
            [Revmob showInterstitial:placementId];
        }
        else
        {
            [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        }
    }
    else if (adConfig.adFormat == VIDEO)
    {
        NSString *placementId = [self getAdPlacementId:adConfig];
        if ([Revmob hasAdCachedOnAdUnit:RMRewardedVideo withPlacement:placementId])
        {
            [Revmob showRewardedVideo:placementId];
        }
        else
        {
            [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        }
    }
}

-(void) fetchAd:(PokktAdConfig *)adConfig withAdView:(UIView *)bannerView withRootViewController:(UIViewController *)rootViewController withDelegate:(id<PokktBannerDelegate>)bannerDelegate onSuccess:(void (^)(id))successCallback onFailure:(void (^)(id))failureHandler
{
    self.bannerDelegate = bannerDelegate;
    self.bannerContainer = bannerView;

    NSString *placementId = [self getAdPlacementId:adConfig];
    if ([Revmob hasAdCachedOnAdUnit:RMBanner withPlacement:placementId])
    {
        [self showBanner:placementId];
    }
    else
    {
        [Revmob cacheBanner:placementId];
    }
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        NSString *placementId = [self getAdPlacementId:adConfig];
        if (placementId != nil && [Revmob hasAdCachedOnAdUnit:RMInterstitial withPlacement:placementId])
        {
            [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:YES];
        }
        else
        {
            [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:NO];
        }
    }
    else if (adConfig.adFormat == VIDEO)
    {
        NSString *placementId = [self getAdPlacementId:adConfig];
        if ([Revmob hasAdCachedOnAdUnit:RMRewardedVideo withPlacement:placementId])
        {
            [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:YES];
        }
        else
        {
            [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:NO];
        }
    }
    
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.intertitialTimeOutDict setValue:[NSNumber numberWithBool:YES] forKey:adConfig.getKey];
    }
    else if (adConfig.adFormat == VIDEO)
    {
        [self.videoTimeOutDict setValue:[NSNumber numberWithBool:YES] forKey:adConfig.getKey];
    }
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == INTERSTITIAL)
    {
        flag = YES;
    }
    else if (adConfig.adFormat == VIDEO)
    {
        flag = YES;
    }
    else if (adConfig.adFormat == BANNER)
    {
        flag = YES;
    }
    
    return flag;
}

-(void)showBanner:(NSString *)placementId
{
    RMBannerView *adView = [Revmob getBanner:placementId];
    [adView setFrame:CGRectMake(0, 0, self.bannerContainer.frame.size.width, self.bannerContainer.frame.size.height)];
    [self.bannerContainer addSubview:adView];
    [self.bannerDelegate bannerLoaded:placementId];
}


-(NSString *)getAdPlacementId:(PokktAdConfig *)adConfig
{
    NSString *placementId = @"";
        
    if ([self.networkModel.customData objectForKey:@"screens_mapping_data"] != nil && [self.networkModel.customData objectForKey:@"screens_mapping_data"])
    {
        NSDictionary *screenDict = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
        
        if ([screenDict objectForKey:adConfig.screenName] != nil && [screenDict objectForKey:adConfig.screenName])
        {
            NSDictionary *networkDict = [screenDict objectForKey:adConfig.screenName];
            
            if ([networkDict objectForKey:@"network_screen"] != nil && [networkDict objectForKey:@"network_screen"])
            {
                placementId = [networkDict objectForKey:@"network_screen"];
            }
        }
        
    }
    else
    {
        placementId = adConfig.screenName;
    }
    if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.intertitialAdConfigDict setObject:adConfig forKey:placementId];
    }
    else if (adConfig.adFormat == VIDEO)
    {
       [self.videoAdConfigDict setObject:adConfig forKey:placementId];
    }
    
    
    return placementId;
}


#pragma mark Non Rewarded Video ads delegates

- (void) revmobDidCacheAd:(RMAdUnits)adUnit withPlacement:(NSString *)placement
{
    if (adUnit == RMInterstitial) {
        PokktAdConfig *adConfig = [self.intertitialAdConfigDict objectForKey:placement];
        if (![[self.intertitialTimeOutDict objectForKey:adConfig.getKey] boolValue])
        {
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
        }
    }
    else if (adUnit == RMRewardedVideo)
    {
        PokktAdConfig *adConfig = [self.videoAdConfigDict objectForKey:placement];
        if (![[self.videoTimeOutDict objectForKey:adConfig.getKey] boolValue])
        {
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
        }
    }
    else if (adUnit == RMBanner)
     {
         [self showBanner:placement];
         [self.bannerDelegate bannerLoaded:placement];
     }
}

- (void) revmobAdClicked:(RMAdUnits)adUnit withPlacement:(NSString *) placement
{
    
}

- (void) revmobAdDisplayed:(RMAdUnits)adUnit withPlacement:(NSString *)placement
{
    if (adUnit == RMInterstitial) {
        PokktAdConfig *adConfig = [self.intertitialAdConfigDict objectForKey:placement];
        [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:adConfig];
    }
    else if (adUnit == RMRewardedVideo)
    {
        PokktAdConfig *adConfig = [self.videoAdConfigDict objectForKey:placement];
        [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:adConfig];
    }
 }

- (void) revmobAdDismissed:(RMAdUnits)adUnit withPlacement:(NSString *)placement
{
    if (adUnit == RMInterstitial) {
        PokktAdConfig *adConfig = [self.intertitialAdConfigDict objectForKey:placement];
         [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:adConfig];
    }
    else if (adUnit == RMRewardedVideo)
    {
        PokktAdConfig *adConfig = [self.videoAdConfigDict objectForKey:placement];
        [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:adConfig];
    }
   
}

- (void) revmobAdFailedToRender:(RMAdUnits)adUnit withPlacement:(NSString *)placement
{
    
}

- (void) revmobUserWillLeaveAplication:(RMAdUnits)adUnit fromPlacement:(NSString* )placement
{
    
}

- (void) revmobAdUnitFailedToCache:(RMAdUnits)adUnit withError:(NSError *)error withPlacement:(NSString *)placement
{
    
    if (adUnit == RMInterstitial) {
        PokktAdConfig *adConfig = [self.intertitialAdConfigDict objectForKey:placement];
        if (![[self.intertitialTimeOutDict objectForKey:adConfig.getKey] boolValue])
        {
            dispatch_async(dispatch_get_main_queue(), ^{
                
                [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:error.description];
            });
        }
    }
    else if (adUnit == RMRewardedVideo)
    {
        PokktAdConfig *adConfig = [self.videoAdConfigDict objectForKey:placement];
        if (![[self.videoTimeOutDict objectForKey:adConfig.getKey] boolValue])
        {
            dispatch_async(dispatch_get_main_queue(), ^{
                
                [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:error.description];
            });
        }
    }
    else if (adUnit == RMBanner)
    {
        [self.bannerDelegate bannerLoadFailed:placement errorMessage:error.description];
    }

   
}

- (void) revmobRewardedVideoActionDidCompleteOnPlacement:(NSString *) placement
{
   
    PokktAdConfig *adConfig = [self.videoAdConfigDict objectForKey:placement];
    [PokktNetworkDelegate onAdCompleted:self.networkModel adConfig:adConfig];
}

@end



