//
//  StartAppNetwork.m
//  PokktAdsDemo
//
//  Created by Pokkt on 03/03/17.
//  Copyright © 2017 Pokkt. All rights reserved.
//

#import "StartAppNetwork.h"

#import "PokktSDK/PokktNetworkDelegate.h"



#define versionNumber @"3.5.0"

@implementation StartAppNetwork

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.networkModel = customData;
    
    self.interstitialAd = [[StartAppInterstitialAds alloc] init];
    [self.interstitialAd initNetworkWithNetworkModel:customData];
    
    self.bannerAd = [[StartAppBanner alloc] init];
    self.bannerAd.networkModel = customData;
    
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.interstitialAd cacheAd:adConfig];
    }
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.interstitialAd showAd:adConfig viewController:viewController];
    }
}

-(void) fetchAd:(PokktAdConfig *)adConfig withAdView:(UIView *)bannerView withRootViewController:(UIViewController *)rootViewController withDelegate:(id<PokktBannerDelegate>)bannerDelegate onSuccess:(void (^)(id))successCallback onFailure:(void (^)(id))failureHandler
{
    [self.bannerAd fetchAd:adConfig withAdView:bannerView withRootViewController:rootViewController withDelegate:bannerDelegate onSuccess:successCallback onFailure:failureHandler];
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.interstitialAd checkAdAvailability:adConfig];
    }
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == INTERSTITIAL)
    {
        [self.interstitialAd setCacheTimedOut:adConfig];
    }
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == INTERSTITIAL)
    {
        flag = [self.interstitialAd checkAdFormatSupport:adConfig];
    }
    else if (adConfig.adFormat == BANNER)
    {
        flag = [self.bannerAd checkAdFormatSupport:adConfig];
    }
    
    return flag;
}

@end

@implementation StartAppInterstitialAds

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    self.networkModel = customData;
    self.cacheLock  = [NSRecursiveLock new];
    NSString *appId = [customData.customData objectForKey:@"appId"];//@"202337895"
    if (appId != nil && appId.length > 0)
    {
        STAStartAppSDK* sdk = [STAStartAppSDK sharedInstance];
        sdk.appID = appId;
        
        startApp = [[STAStartAppAd alloc] init];
    }
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if (adConfig.isRewarded)
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:self.adConfig errorMessage:@"Invalid Network"];
        return;
    }
    
    if ([self.cacheLock tryLock])
    {
        self.adConfig = adConfig;
        if ([startApp isReady])
        {
            [self.cacheLock unlock];
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
            return;
        }
        
        [startApp loadAdWithDelegate:self];
 
    }
    else
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"RevMob Ad is not available"];
    }
    }

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (adConfig.isRewarded)
    {
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        return;
    }
    self.adConfig = adConfig;
    if ([startApp isReady])
    {
         [startApp showAd];
    }
    else
    {
         [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }
   
}


-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;
    if ([startApp isReady])
    {
        flag = YES;
    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:self.adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    self.isTimeOut = YES;
}


-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == INTERSTITIAL)
    {
        flag = YES;
    }
    
    return flag;
}

- (void) didLoadAd:(STAAbstractAd*)ad
{
    if (!self.isTimeOut)
    {
        [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:self.adConfig rewardValue:0 downLoadTime:@""];
    }
}

- (void) failedLoadAd:(STAAbstractAd*)ad withError:(NSError *)error
{
    [self.cacheLock unlock];
    if (!self.isTimeOut)
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:self.adConfig errorMessage:error.description];
    }

}

- (void) didShowAd:(STAAbstractAd*)ad
{
    [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:self.adConfig];
}

- (void) failedShowAd:(STAAbstractAd*)ad withError:(NSError *)error
{
    [self.cacheLock unlock];
    [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:self.adConfig];
}

- (void) didCloseAd:(STAAbstractAd*)ad
{
    [self.cacheLock unlock];
    [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:self.adConfig];
}

- (void) didClickAd:(STAAbstractAd*)ad
{
    
}

- (void) didCloseInAppStore:(STAAbstractAd*)ad
{
    
}

- (void) didCompleteVideo:(STAAbstractAd*)ad
{
    [PokktNetworkDelegate onAdCompleted:self.networkModel adConfig:self.adConfig];
}


@end

@implementation StartAppBanner

-(void) fetchAd:(PokktAdConfig *)adConfig withAdView:(UIView *)bannerView withRootViewController:(UIViewController *)rootViewController withDelegate:(id<PokktBannerDelegate>)bannerDelegate onSuccess:(void (^)(id))successCallback onFailure:(void (^)(id))failureHandler
{
    NSLog(@"StartApp banner fetch request");
    self.bannerDelegate = bannerDelegate;
    self.rootViewController = rootViewController;

    STABannerView  *banner = [[STABannerView alloc] initWithSize:STA_AutoAdSize autoOrigin:STAAdOrigin_Top
                                                withView:bannerView withDelegate:self];
    [bannerView addSubview:banner];
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == BANNER)
    {
        return true;
    }
    return false;
}

- (void) didDisplayBannerAd:(STABannerView*)banner
{
    [self.bannerDelegate bannerLoaded:@"StartApp Banner loaded"];
}

- (void) failedLoadBannerAd:(STABannerView*)banner withError:(NSError *)error
{
    [self.bannerDelegate bannerLoadFailed:@"" errorMessage:error.description];
}

- (void) didClickBannerAd:(STABannerView*)banner
{
    
}

- (void) didCloseBannerInAppStore:(STABannerView*)banner
{
    
}



@end

