//
//  SuperSonicNetwork.m
//  SampleApp
//
//  Created by Pokkt on 10/26/15.
//  Copyright © 2015 Pokkt. All rights reserved.
//

#import "SuperSonicNetwork.h"
#import <Supersonic/Supersonic.h>
#import <Supersonic/SupersonicConfiguration.h>
#import <Supersonic/SUSupersonicAdsConfiguration.h>
#import "PokktSDK/PokktNetworkDelegate.h"

#define versionNumber @"6.4.21"

@implementation SuperSonicNetwork

static BOOL isInitialized;

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    if (isInitialized) {
        return;
    }
    isInitialized = YES;
    self.networkModel = customData;
    
    self.rewardedAd = [[SuperSonicRewardedAd alloc] init];
    [self.rewardedAd initNetworkWithNetworkModel:customData];
    
    self.interstitialAd = [[SuperSonicInterstitialAd alloc] init];
    [self.interstitialAd initNetworkWithNetworkModel:customData];
    if ([PokktDebugger isDebugEnabled])
    {
        [[Supersonic sharedInstance] setLogDelegate:self];
    }
    
    
    NSLog(@"[SuperSonic SDK]SuperSonic Initialised with Timer");
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    
    NSString *str = nil;
   
    if (adConfig.isRewarded)
    {
        [self.rewardedAd cacheAd:adConfig];
    }
    else
    {
        [self.interstitialAd cacheAd:adConfig];
    }
    NSLog(@"%@",str);
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (adConfig.isRewarded)
    {
        [self.rewardedAd showAd:adConfig viewController:viewController];
    }
    else
    {
        [self.interstitialAd showAd:adConfig viewController:viewController];
    }
   
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    if (adConfig.isRewarded)
    {
        [self.rewardedAd checkAdAvailability:adConfig];
    }
    else
    {
        [self.interstitialAd checkAdAvailability:adConfig];
    }
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    if (adConfig.isRewarded)
    {
        [self.rewardedAd setCacheTimedOut:adConfig];
    }
    else
    {
        [self.interstitialAd setCacheTimedOut:adConfig];
    }

    NSLog(@"[SuperSonic SDK] Caching time out");
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == 0)
    {
        return [self.rewardedAd checkAdFormatSupport:adConfig];
    }
    else if (adConfig.adFormat == 3)
    {
        return [self.interstitialAd checkAdFormatSupport:adConfig];
    }
        
    
    return false;
}


- (void)sendLog:(NSString *)log level:(LogLevel)level tag:(LogTag)tag
{
    [PokktDebugger printLog:log];
}


@end

@interface SuperSonicRewardedAd()
@property(atomic)BOOL isCachingAvailable;
@property(atomic,strong) NSString *screenName;

@end
@implementation SuperSonicRewardedAd

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    NSString *userId = @"uniqueId";
    NSString *appKey = [customData.customData objectForKey:@"appKey"];
    self.network = customData;
    
    [[Supersonic sharedInstance] setRVDelegate:self];
    [[Supersonic sharedInstance] initRVWithAppKey:appKey withUserId:userId];

    self.screenName = nil;
    self.isCachingAvailable = YES;
    
    if ([PokktDebugger isDebugEnabled])
    {
        [[Supersonic sharedInstance] setLogDelegate:self];
    }

    
    NSLog(@"[SuperSonic SDK]SuperSonic Initialised with Timer");
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    
    NSString *str = nil;
    if (self.screenName != nil && ![self.screenName isEqualToString:adConfig.screenName])
    {
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig
                                     errorMessage:@"Caching in Fyber Failed"];
        
        return;
    }
    
    if (adConfig.isRewarded)
    {
        if (self.isCachingAvailable)
        {
            self.isCachingAvailable = NO;
            self.screenName = adConfig.screenName;

            BOOL flag = [[Supersonic sharedInstance] isAdAvailable];
            
            if (flag)
            {
                [PokktNetworkDelegate didFinishedAdDownload:self.network adConfig:adConfig rewardValue:0 downLoadTime:@""];
                str = @"[SuperSonic SDK] SuperSonic video is available to watch";
            }else
            {
                str = @"[SuperSonic SDK] SuperSonic video is not available right now!!";
                [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:str];
            }
            
        }
        else
        {
            str = @"[SuperSonic SDK] SuperSonic video is not available right now!!";
            [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:str];
        }
        
    }
    NSLog(@"%@",str);
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (self.screenName == nil || ![self.screenName isEqualToString:adConfig.screenName])
    {
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
        return;
    }
    
    if (adConfig.isRewarded)
    {
        BOOL flag = [[Supersonic sharedInstance] isAdAvailable];
        
        if (flag)
        {
            self.adConfig = adConfig;
            dispatch_async(dispatch_get_main_queue(), ^{
                [[Supersonic sharedInstance] showRV];
            });
            NSLog(@"[SuperSonic SDK]Supersonic Video Play");
        }
        else
        {
            [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
            NSLog(@"[SuperSonic SDK]Supersonic Video failed to Play");
        }
        
    }
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    if (self.screenName == nil || ![self.screenName isEqualToString:adConfig.screenName])
    {
        [PokktNetworkDelegate onAdAvailabilityStatus:self.network adConfig:adConfig isAdAvailable:NO];
        return;
    }
    
    BOOL flag = NO ;
    if (adConfig.isRewarded)
    {
        if ([[Supersonic sharedInstance] isAdAvailable])
        {
            flag = YES;
        }
    }

    [PokktNetworkDelegate onAdAvailabilityStatus:self.network adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"[SuperSonic SDK] Caching time out");
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.network;
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == 0)
    {
        return true;
    }
    
    return false;
}


#pragma mark SuperSonic Video Ad Delegates

- (void)supersonicRVInitSuccess{
    NSLog(@"SuperSonic SDK|%s|%s", "DemoRVDelegate", "supersonicRVInitSuccess");
}

- (void)supersonicRVInitFailedWithError:(NSError *)error{
    NSLog(@"SuperSonic SDK|%s|%s|%@", "DemoRVDelegate", "supersonicRVInitFailedWithError", [error description]);
}

- (void)supersonicRVAdAvailabilityChanged:(BOOL)hasAvailableAds{
    if (hasAvailableAds == TRUE)
    {
        NSLog(@"[SuperSonic SDK] SuperSonic Video downloaded");
    }
    else {
        NSLog(@"[SuperSonic SDK] SuperSonic Video Not available");
    }
}

- (void)supersonicRVAdOpened{
    NSLog(@"SuperSonic SDK|%s|%s", "DemoRVDelegate", "supersonicRVAdOpened");
    [PokktNetworkDelegate onAdDisplayed:self.network adConfig:self.adConfig];
}

- (void)supersonicRVAdStarted{
    NSLog(@"SuperSonic SDK|%s|%s", "DemoRVDelegate", "supersonicRVAdStarted");
}

- (void)supersonicRVAdEnded{
    NSLog(@"SuperSonic SDK|%s|%s", "DemoRVDelegate", "supersonicRVAdEnded");
}

- (void)supersonicRVAdClosed{
    NSLog(@"SuperSonic SDK|%s|%s", "DemoRVDelegate", "supersonicRVAdClosed");
    [PokktNetworkDelegate onAdClosed:self.network adConfig:self.adConfig];
    self.isCachingAvailable = YES;
    self.screenName = nil;
}

- (void)supersonicRVAdRewarded:(SupersonicPlacementInfo*)placementInfo{
    NSLog(@"SuperSonic SDK|%s|%s|placementInfo = %@", "DemoRVDelegate", "supersonicRVAdRewarded", placementInfo);
    [PokktNetworkDelegate onAdCompleted:self.network adConfig:self.adConfig];
    
    if (self.adConfig.isRewarded) {
        float amount = 0.0;
        if ([[self.network.customData objectForKey:@"rewardAmount"] floatValue]) {
            amount = [[self.network.customData objectForKey:@"rewardAmount"] floatValue];
        }
        [PokktNetworkDelegate onAdGratified:self.network adConfig:_adConfig rewardPoint:amount];
    }
    
}

- (void)supersonicRVAdFailedWithError:(NSError *)error
{
    NSLog(@"[SuperSonic SDK] SuperSonic Video download failed reason: %@",error.description);
    self.isCachingAvailable = YES;
    self.screenName = nil;
}

- (void)sendLog:(NSString *)log level:(LogLevel)level tag:(LogTag)tag
{
    [PokktDebugger printLog:log];
}


@end


@interface SuperSonicInterstitialAd()

@property(atomic,strong) NSString *screenName;
@property(atomic)BOOL isCachingAvailable;
@end

@implementation SuperSonicInterstitialAd

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
   NSString *userId = @"uniqueID";
   NSString *appKey = [customData.customData objectForKey:@"appKey"];
    self.network = customData;
    
    [[Supersonic sharedInstance] setISDelegate:self];
    [[Supersonic sharedInstance] initISWithAppKey:appKey withUserId:userId];
    
    self.screenName = nil;
    self.isCachingAvailable = YES;
    
    if ([PokktDebugger isDebugEnabled])
    {
        [[Supersonic sharedInstance] setLogDelegate:self];
    }

    NSLog(@"[SuperSonic SDK]SuperSonic Initialised with Timer");
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if (self.screenName != nil && ![self.screenName isEqualToString:adConfig.screenName])
    {
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig
                                     errorMessage:@"Caching in Fyber Failed"];
        
        return;
    }
    
    NSString *str = nil;
    
    if (!adConfig.isRewarded)
    {
        if (self.isCachingAvailable)
        {
            self.isCachingAvailable = NO;
            self.screenName = adConfig.screenName;

            BOOL flag = [[Supersonic sharedInstance] isInterstitialAvailable];
            
            if (flag)
            {
                [PokktNetworkDelegate didFinishedAdDownload:self.network adConfig:adConfig rewardValue:0 downLoadTime:@""];
                str = @"[SuperSonic SDK] SuperSonic Interstitial is available to watch";
            }
            else
            {
                str = @"[SuperSonic SDK] SuperSonic Interstitial is not available right now!!";
                [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:str];
            }
        }
        else
        {
            str = @"[SuperSonic SDK] SuperSonic Interstitial is not available right now!!";
            [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:str];
        }
    }
    NSLog(@"%@",str);
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    if (self.screenName == nil || ![self.screenName isEqualToString:adConfig.screenName])
    {
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
        return;
    }
    
    if (!adConfig.isRewarded)
    {
        BOOL flag = [[Supersonic sharedInstance] isInterstitialAvailable];
        
        if (flag)
        {
            self.adConfig = adConfig;
             viewController = [[[UIApplication sharedApplication] keyWindow] rootViewController];
            dispatch_async(dispatch_get_main_queue(), ^{
                
                [[Supersonic sharedInstance] showISWithViewController:viewController];
            });
            NSLog(@"[SuperSonic SDK]Interstitial Ad Display");
        }
        else
        {
            [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
            NSLog(@"[SuperSonic SDK]Interstitial Ad failed to Display");
        }
    }
    
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    if (self.screenName == nil || ![self.screenName isEqualToString:adConfig.screenName])
    {
        [PokktNetworkDelegate onAdAvailabilityStatus:self.network adConfig:adConfig isAdAvailable:NO];
        return;
    }
    
    BOOL flag = NO ;
    if (!adConfig.isRewarded)
    {
        if ([[Supersonic sharedInstance] isInterstitialAvailable])
        {
            flag = YES;
        }
        
    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.network adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"[SuperSonic SDK] Caching time out");
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.network;
}

- (BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    if (adConfig.adFormat == 3)
    {
        return true;
    }
    
    return false;
}



#pragma mark SuperSonic Interstitial Ad Delegates

- (void)supersonicISInitSuccess
{
    NSLog(@"SuperSonic SDK|%s|%s", "ISDelegate", "supersonicISInitSuccess");
}

- (void)supersonicISReady
{
    NSLog(@"Ad is ready");
}

- (void)supersonicISInitFailedWithError:(NSError *)error
{
    NSLog(@"SuperSonic SDK|%s|%s|%@", "ISDelegate", "supersonicISInitFailedWithError", [error description]);
}

- (void)supersonicISShowSuccess
{
    
}

- (void)supersonicISShowFailWithError:(NSError *)error
{
    NSLog(@"[SuperSonic SDK] SuperSonic Interstitial download failed reason: %@",error.description);
    self.isCachingAvailable = YES;
    self.screenName = nil;
}


- (void)supersonicISAdClicked
{
    
}

- (void)supersonicISAdClosed
{
    [PokktNetworkDelegate onAdCompleted:self.network adConfig:self.adConfig];
    [PokktNetworkDelegate onAdClosed:self.network adConfig:self.adConfig];
    NSLog(@"[SuperSonic SDK] SuperSonic Interstitial closed");
    
    self.isCachingAvailable = YES;
    self.screenName = nil;
}

- (void)sendLog:(NSString *)log level:(LogLevel)level tag:(LogTag)tag
{
    [PokktDebugger printLog:log];
}


@end
