//
//  TapjoyNetwork.m
//  SampleApp
//
//  Created by Subhendu Sekhar Sahu on 16/11/15.
//  Copyright © 2015 Pokkt. All rights reserved.
//

#import "TapjoyNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"

#define versionNumber @"11.11.0"

@implementation TapjoyNetwork

static NSString *networkType = @"TapJoy";

- (instancetype)init
{
    self = [super init];
    if (self)
    {
        
    }
    return self;
}

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    NSLog(@"[TAP-JOY-POKKT] %@ is initialised", networkType);
    
    self.networkModel  =customData;
    self.adConfigDictionary = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.placementDictionary = [[NSMutableDictionary alloc] initWithCapacity:0];
    self.timeoutDictionary = [[NSMutableDictionary alloc] initWithCapacity:0];
    
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(tjcConnectSuccess:)
                                                 name:TJC_CONNECT_SUCCESS
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(tjcConnectFail:)
                                                 name:TJC_CONNECT_FAILED
                                               object:nil];
    
    NSString * sdkKey = [customData.customData objectForKey:@"sdkKey"];
    
    [Tapjoy setDebugEnabled:[PokktDebugger isDebugEnabled]];
    
    //Tapjoy connect call
    [Tapjoy connect:sdkKey];
    
    [Tapjoy setVideoAdDelegate:self];
   // [Tapjoy setViewDelegate:self];
    
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    if(!adConfig.isRewarded)
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"[TAP-JOY SDK]Invalid Network"];
        return;
    }
    
    TJPlacement *placement = [self getPlacementWithAdConfig:adConfig];
    
    if (placement == nil)
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"[Tapjoy SDK ]placement content is not available"];
        return;
    }
    
    [self.placementDictionary setObject:placement forKey:adConfig.getKey];
    [self.adConfigDictionary setObject:adConfig forKey:placement.placementName];
    
    [Tapjoy getCurrencyBalanceWithCompletion:^(NSDictionary *parameters, NSError *error)
     {
         if (error)
         {
             NSLog(@"[TAP-JOY-POKKT] Error in getCurrencybalance in Tapjoy caching time %@", error.description);
         }
         else
         {
             self.currentCurrencyBalance = [parameters[@"amount"] intValue];
             NSLog(@"[TAP-JOY-POKKT] Current Balance While caching %d", self.currentCurrencyBalance);
             
         }
     }];
    
    
    if ([placement isContentReady])
    {
        NSLog(@"[TAP-JOY-POKKT] %@ video cached previously", networkType);
        
        [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
    }
    else
    {
        NSLog(@"[TAP-JOY-POKKT] %@ video caching started", networkType);
        
        [self.timeoutDictionary setObject:[NSNumber numberWithBool:NO] forKey:adConfig.getKey];
        [placement requestContent];
    }
    
}

- (TJPlacement *)getPlacementWithAdConfig: (PokktAdConfig *)adConfig
{
    TJPlacement *placement = nil;
    
    if ([self.placementDictionary objectForKey:adConfig.getKey])
    {
        placement = [self.placementDictionary objectForKey:adConfig.getKey];
    }
    else
    {
        NSDictionary *screenMapdictionary = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
        
        if (screenMapdictionary != nil && [screenMapdictionary objectForKey:adConfig.screenName])
        {
            NSDictionary * networkScreenDetails = [screenMapdictionary objectForKey:adConfig.screenName];
            
            NSString * placementName = [networkScreenDetails objectForKey:@"network_screen"];
            BOOL isRewarded = [[networkScreenDetails objectForKey:@"incent"] boolValue];
            BOOL isNonRewarded = [[networkScreenDetails objectForKey:@"non_incent"] boolValue];
            
            if (placementName != nil && placementName.length !=0)
            {
                if ((isRewarded && adConfig.isRewarded) || (isNonRewarded && !adConfig.isRewarded))
                {
                    placement = [TJPlacement placementWithName:placementName delegate:self];
                }
            }
            else
            {
                placement = [TJPlacement placementWithName:adConfig.screenName delegate:self];
            }
        }
        else
        {

            placement = [TJPlacement placementWithName:adConfig.screenName delegate:self];
        }
        
    }
    
    return placement;
}

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    NSLog(@"[TAP-JOY-POKKT] %@ play video called", networkType);
    
    self.viewController = [[[UIApplication sharedApplication] keyWindow] rootViewController];
    
    if(!adConfig.isRewarded)
    {
        [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"[TAP-JOY SDK]Invalid Network"];
        return;
    }
    
    TJPlacement *placement = [self getPlacementWithAdConfig:adConfig];
    
    if ([placement isContentReady] && placement != nil)
    {
        self.currentPlayAdConfig = adConfig;
        dispatch_async(dispatch_get_main_queue(), ^{
            [placement showContentWithViewController: self.viewController];
        });
    }
    else
    {
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;
    
    if(adConfig.isRewarded)
    {
        TJPlacement *placement = [self getPlacementWithAdConfig:adConfig];
        
        if (placement != nil && [placement isContentReady])
        {
            flag = YES;
        }
    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"[TAP-JOY SDK] Caching time out");
    [self.timeoutDictionary setObject:[NSNumber numberWithBool:YES] forKey:adConfig.getKey];
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag;
    
    if (adConfig.adFormat == 0)
    {
        flag = true;
    }
    
    return flag;
}

-(void)tjcConnectSuccess:(NSNotification*)notifyObj
{
    NSLog(@"[TAP-JOY-POKKT] Tapjoy connect Succeeded");
}

-(void)tjcConnectFail:(NSNotification*)notifyObj
{
    NSLog(@"[TAP-JOY-POKKT] Tapjoy connect Failed");
}


#pragma mark Tapjoy Video

- (void)videoAdCompleted
{
    NSLog(@"[TAP-JOY-POKKT]  %@ videoAdCompleted", networkType);
    [PokktNetworkDelegate onAdCompleted:self.networkModel adConfig:self.currentPlayAdConfig];
}

#pragma mark TJPlacementDelegate methods

- (void)requestDidSucceed:(TJPlacement*)placement
{
    NSLog(@"[TAP-JOY-POKKT] %@", [NSString stringWithFormat : @"Tapjoy request content complete, isContentAvailable:%d", placement.isContentAvailable]);
    
    if (placement.isContentAvailable == 0)
    {
        PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:placement.placementName];
        
        if (adConfig)
        {
            if ([[self.timeoutDictionary objectForKey:adConfig.getKey] boolValue])
            {
                NSLog(@"[TAP-JOY-POKKT] %@ video caching failed after timeout", networkType);
            }
            else
            {
                NSLog(@"[TAP-JOY-POKKT] %@ video download failed", networkType);
                
                [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"[Tapjoy SDK ]placement content is not available"];
            }
        }
        else
        {
            NSLog(@"Unknown placement");
        }
        
    }
}

- (void)contentIsReady:(TJPlacement*)placement
{
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:placement.placementName];
    
    if (adConfig)
    {
        if ([[self.timeoutDictionary objectForKey:adConfig.getKey] boolValue])
        {
            NSLog(@"[TAP-JOY-POKKT] %@ video caching successful after timeout", networkType);
        }
        else
        {
            NSLog(@"[TAP-JOY-POKKT] %@ video caching successful before timeout", networkType);
            
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
        }
    }
    else
    {
        NSLog(@"Unknown placement");
    }
}

- (void)requestDidFail:(TJPlacement*)placement error:(NSError *)error
{
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:placement.placementName];
    
    if (adConfig)
    {
        if ([[self.timeoutDictionary objectForKey:adConfig.getKey] boolValue])
        {
            NSLog(@"[TAP-JOY-POKKT] %@ video caching failed after timeout", networkType);
        }
        else
        {
            NSLog(@"[TAP-JOY-POKKT] %@ video download failed", networkType);
            
            [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:error.description];
        }
    }
    else
    {
        NSLog(@"Unknown placement");
    }
}

- (void)contentDidAppear:(TJPlacement*)placement
{
    NSLog(@"[TAP-JOY-POKKT] Content did appear for %@ placement", [placement placementName]);

    [Tapjoy getCurrencyBalance];
    
    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:placement.placementName];
    
    if (adConfig)
    {
        [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:adConfig];
    }
    else
    {
        NSLog(@"Unknown placement");
    }
}

- (void)contentDidDisappear:(TJPlacement*)placement
{
    NSLog(@"[TAP-JOY-POKKT] Content did disappear for %@ placement", [placement placementName]);
    
    [Tapjoy getCurrencyBalanceWithCompletion:^(NSDictionary *parameters, NSError *error)
     {
         if (error)
             NSLog(@"[TAP-JOY-POKKT] Error in getCurrencybalance in Tapjoy %@", error.description);
         else
         {
             int currentAmount = [parameters[@"amount"] intValue];
             
             NSLog(@"[TAP-JOY-POKKT] Current Balance While Disappearing %d", self.currentCurrencyBalance);
             
             
             int reward = currentAmount - self.currentCurrencyBalance;
             self.currentCurrencyBalance = currentAmount;
             if (self.currentPlayAdConfig.isRewarded)
             {
             [PokktNetworkDelegate onAdGratified:self.networkModel adConfig:self.currentPlayAdConfig rewardPoint:reward];
             }
         }
     }];

    PokktAdConfig *adConfig = [self.adConfigDictionary objectForKey:placement.placementName];
    
    if (adConfig)
    {
        [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:adConfig];
    }
    else
    {
        NSLog(@"Unknown placement");
    }

}

@end
