//
//  UnityAdsNetwork.m
//  SampleApp
//
//  Created by Pokkt on 01/12/15.
//  Copyright © 2015 Pokkt. All rights reserved.
//

#import "UnityAdsNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"

#define versionNumber @"2.1.1"

@implementation UnityAdsNetwork

static bool isInitialized;

#pragma mark -
#pragma mark - PokktAdNetwork Protocol Method

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    NSLog(@"Required Minimum Unity-Ads version is: %@, found: %@", @"2.0+", [UnityAds getVersion]);
    
    if (isInitialized)
    {
        NSLog(@"[UnityAds SDK] UnityAds already initialised!");
        return;
    }
    
    self.network = customData;
    
    NSString *gameId = [customData.customData objectForKey:@"gameId"];
    
    rewardAmount = [NSString stringWithFormat:@"%d",[[customData.customData objectForKey:@"rewardAmount"] intValue]];
        
    self.playerVC = [[[UIApplication sharedApplication] keyWindow] rootViewController];
    [UnityAds initialize:gameId delegate:self];
    [UnityAds setDebugMode:[PokktDebugger isDebugEnabled]];
    isInitialized = YES;
    
    NSLog(@"[UnityAds SDK] UnityAds Initialised");
}

- (void) cacheAd:(PokktAdConfig *)adConfig
{
    NSString *placementId = [self getPlacementIdWithAdConig:adConfig];
    
    if (placementId != nil && placementId.length > 0)
    {
        [self cacheAdWithPlacementId:placementId PokktAdConfig:adConfig];
    }
    else
    {
        NSString *errorMsg = (@"[UnityAds SDK] UnityAds video is not available right now!!");
        NSLog(@"%@", errorMsg);
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:errorMsg];
    }
 }

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    NSString *placementId = [self getPlacementIdWithAdConig:adConfig];
    
    self.playerVC = [[[UIApplication sharedApplication] keyWindow] rootViewController];;
    
    if (placementId != nil && placementId.length >0 && [UnityAds isReady:placementId])
    {
        self.playAdConfig = adConfig;
        
        [UnityAds show:self.playerVC placementId:placementId];
        
        NSLog(@"[UnityAds SDK] UnityAds Video Play");
    }
    else
    {
        [PokktNetworkDelegate didFailedToShowAd:self.network adConfig:adConfig];
    }
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;
    
    NSString *placementId = [self getPlacementIdWithAdConig:adConfig];
    
    if (placementId != nil && placementId.length >0 &&[UnityAds isReady:placementId])
    {
        flag = YES;
    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.network adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"[UnityAds SDK] Caching time out");
}

- (PokktNetworkModel *)getNetworkModel
{
    return self.network;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag;
    
    if (adConfig.adFormat == VIDEO)
    {
        flag = true;
    }
    
    return flag;
}


#pragma mark - 
#pragma mark - Internal Methods

-(void)cacheAdWithPlacementId: (NSString *)placementId PokktAdConfig:(PokktAdConfig *)adConfig
{
    NSString *msgStr = nil;
    
    if ([UnityAds isReady:placementId])
    {
        [PokktNetworkDelegate didFinishedAdDownload:self.network adConfig:adConfig rewardValue:rewardAmount.floatValue downLoadTime:@""];
        msgStr = @"[UnityAds SDK] UnityAds video is available to watch";
    }
    else
    {
        msgStr = @"[UnityAds SDK] UnityAds video is not available right now!!";
        [PokktNetworkDelegate didFailedAdDownload:self.network adConfig:adConfig errorMessage:msgStr];
    }
    NSLog(@"%@",msgStr);
    
}

-(NSString *)getPlacementIdWithAdConig:(PokktAdConfig *)adConfig
{
    NSString *placementId = nil;
    
    if ([self.network.customData objectForKey:@"screens_mapping_data"] != nil && [self.network.customData objectForKey:@"screens_mapping_data"])
    {
        NSDictionary *screenDict = [self.network.customData objectForKey:@"screens_mapping_data"];
        
        if ([screenDict objectForKey:adConfig.screenName] != nil && [screenDict objectForKey:adConfig.screenName])
        {
            NSDictionary *networkDict = [screenDict objectForKey:adConfig.screenName];
            
            if ([networkDict objectForKey:@"network_screen"] != nil && [networkDict objectForKey:@"network_screen"])
            {
                placementId = [networkDict objectForKey:@"network_screen"];
            }
        }
    }
    else
    {
        placementId = adConfig.screenName;
    }
    
    return placementId;
}



- (void)unityAdsReady:(NSString *)placementId
{
    NSLog(@"%@",[NSString stringWithFormat:@"Unity ads is ready for placementId is %@",placementId]);
}

- (void)unityAdsDidError:(UnityAdsError)error withMessage:(NSString *)message
{
   NSLog(@"%@",[NSString stringWithFormat:@"Unity ads failed with error %@",message]);
}

- (void)unityAdsDidStart:(NSString *)placementId
{
    NSLog(@"Unity Video Started");
    [PokktNetworkDelegate onAdDisplayed:self.network adConfig:self.playAdConfig];
}

- (void)unityAdsDidFinish:(NSString *)placementId
          withFinishState:(UnityAdsFinishState)state
{
    if (state == kUnityAdsFinishStateSkipped)
    {
        [PokktNetworkDelegate onAdSkipped:self.network adConfig:self.playAdConfig];
    }
    else if (state == kUnityAdsFinishStateError)
    {
        
    }
    if (state == kUnityAdsFinishStateCompleted)
    {
        [PokktNetworkDelegate onAdCompleted:self.network adConfig:self.playAdConfig];
    
        if (self.playAdConfig.isRewarded && rewardAmount.floatValue > 0)
        {
            [PokktNetworkDelegate onAdGratified:self.network adConfig:self.playAdConfig rewardPoint:[rewardAmount floatValue]];
        }
     
    }
    
    [PokktNetworkDelegate onAdClosed:self.network adConfig:self.playAdConfig];

}


@end
