//
//  VungalNetwork.m
//  SampleApp
//
//  Created by Subhendu Sekhar Sahu on 01/12/15.
//  Copyright © 2015 Pokkt. All rights reserved.
//

#import "VungleNetwork.h"
#import "PokktSDK/PokktNetworkDelegate.h"

#define versionNumber @"5.1.1"

@interface VungleNetwork ()

@property (nonatomic, strong) VungleSDK *vungleSdk;

@end

@implementation VungleNetwork

static NSString *networkType = @"Vungle";

- (instancetype)init
{
    self = [super init];
    if (self)
    {
        
    }
    return self;
}

- (void) initNetworkWithNetworkModel:(PokktNetworkModel *)customData
{
    NSLog(@"%@ is initialised", networkType);
        
    self.networkModel  =customData;
    
    NSString * appId = [customData.customData objectForKey:@"appId"];
    self.vungleSdk = [VungleSDK sharedSDK];
    [self.vungleSdk setLoggingEnabled:[PokktDebugger isDebugEnabled]];
    [self.vungleSdk setDelegate:self];
    NSArray *placementIds = [self getAllPlacementIds:customData];
    if (placementIds != nil && placementIds.count > 0)
    {
         [self.vungleSdk startWithAppId:appId placements:placementIds error:nil];
    }
   
    
}

- (void) cacheAd: (PokktAdConfig *)adConfig
{
    
    NSString *placementId = [self getAdPlacementId:adConfig];
    if (placementId != nil && placementId.length > 0)
    {
        if( [self.vungleSdk isAdCachedForPlacementID:placementId])
        {
            [PokktNetworkDelegate didFinishedAdDownload:self.networkModel adConfig:adConfig rewardValue:0 downLoadTime:@""];
        }
        else
        {
            [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"Vungle Ad is not available"];
        }
    }
    else
    {
       [PokktNetworkDelegate didFailedAdDownload:self.networkModel adConfig:adConfig errorMessage:@"Invalid Vungle placement id.."];
    }
    

 }

- (void) showAd: (PokktAdConfig *)adConfig viewController:(UIViewController *)viewController
{
    
    NSLog(@"%@ play video called", networkType);
    self.viewController = viewController;
    
    self.adConfig = adConfig;
    NSString *placementId = [self getAdPlacementId:adConfig];
    if (placementId != nil && placementId.length > 0)
    {
        NSError *error;
        if( [self.vungleSdk isAdCachedForPlacementID:placementId])
        {
            
            NSDictionary* options = @{VunglePlayAdOptionKeyUser:@"test_user_id",
                                      VunglePlayAdOptionKeyIncentivizedAlertBodyText : @"If the video isn't completed you won't get your reward! Are you sure you want to close early?",
                                      VunglePlayAdOptionKeyIncentivizedAlertCloseButtonText : @"Close",
                                      VunglePlayAdOptionKeyIncentivizedAlertContinueButtonText : @"Keep Watching",
                                      VunglePlayAdOptionKeyIncentivizedAlertTitleText : @"Careful!",
                                      VunglePlayAdOptionKeyUser:@"uniqueID"};
            
            
            [self.vungleSdk playAd:self.viewController options:options placementID:placementId error:&error];
        }
        else
        {
            [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
        }
        
        if (error)
        {
            NSLog(@"Error encountered playing ad: %@", error);
        }
    }
    else
    {
        [PokktNetworkDelegate didFailedToShowAd:self.networkModel adConfig:adConfig];
    }
    
    
}

-(void)checkAdAvailability:(PokktAdConfig *)adConfig
{
    BOOL flag = NO;

    NSLog(@"%@ video caching started", networkType);

    NSString *placementId = [self getAdPlacementId:adConfig];
    if (placementId != nil && placementId.length > 0)
    {
        if( [self.vungleSdk isAdCachedForPlacementID:placementId] && adConfig.isRewarded)
        {
            flag = YES;
        }
    }
    
    [PokktNetworkDelegate onAdAvailabilityStatus:self.networkModel adConfig:adConfig isAdAvailable:flag];
}

- (void) setCacheTimedOut: (PokktAdConfig *)adConfig
{
    NSLog(@"[VungleNetwork SDK] Caching time out");
 }

- (PokktNetworkModel *)getNetworkModel
{
    return self.networkModel;
}

-(BOOL) checkAdFormatSupport:(PokktAdConfig *)adConfig
{
    BOOL flag = false;
    
    if (adConfig.adFormat == VIDEO)
    {
        flag = true;
    }
    
    return flag;
}

#pragma mark Get ScreenName from Custom Data

-(NSString *)getAdPlacementId:(PokktAdConfig *)adConfig
{
    NSString *screenName = @"";
    
    if ([self.networkModel.customData objectForKey:@"screens_mapping_data"] != nil && [self.networkModel.customData objectForKey:@"screens_mapping_data"])
    {
        NSDictionary *screenDict = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
        
        if ([screenDict objectForKey:adConfig.screenName] != nil && [screenDict objectForKey:adConfig.screenName])
        {
            NSDictionary *networkDict = [screenDict objectForKey:adConfig.screenName];
            
            if ([networkDict objectForKey:@"network_screen"] != nil && [networkDict objectForKey:@"network_screen"])
            {
                screenName = [networkDict objectForKey:@"network_screen"];
            }
        }
    }
    else
    {
        screenName = adConfig.screenName;
    }
    
    return screenName;
}

-(NSMutableArray *)getAllPlacementIds:(PokktNetworkModel *)networkModel
{
    NSMutableArray *placementIds = [[NSMutableArray alloc] initWithCapacity:0];
    if ([self.networkModel.customData objectForKey:@"screens_mapping_data"] != nil && [self.networkModel.customData objectForKey:@"screens_mapping_data"])
    {
        NSDictionary *screenDict = [self.networkModel.customData objectForKey:@"screens_mapping_data"];
        
        for(NSDictionary *dict in [screenDict allValues])
        {
            if ([dict objectForKey:@"network_screen"] != nil && [[dict objectForKey:@"network_screen"] length] > 0)
            {
                [placementIds addObject:[dict objectForKey:@"network_screen"]];
            }
            
        }
    }
    return placementIds;
    
}



#pragma mark -
#pragma mark - VungleSDK Delegate
#pragma mark - 

- (void)vungleAdPlayabilityUpdate:(BOOL)isAdPlayable placementID:(nullable NSString *)placementID
{
    if (isAdPlayable)
    {
        NSLog(@"%@ ad is available for playback", networkType);
    }
    else
    {
        NSLog(@"%@ ads currently available for playback", networkType);
    }
}

- (void)vungleWillShowAdForPlacementID:(nullable NSString *)placementID
{
    NSLog(@"%@ ad is about to be played!", networkType);
    //Use this delegate method to pause animations, sound, etc.
    [PokktNetworkDelegate onAdDisplayed:self.networkModel adConfig:self.adConfig];
}

- (void)vungleWillCloseAdWithViewInfo:(nonnull VungleViewInfo *)info placementID:(nonnull NSString *)placementID
{
    if (info.completedView)
    {
        [PokktNetworkDelegate onAdCompleted:self.networkModel adConfig:self.adConfig];
    }
    else
    {
        [PokktNetworkDelegate onAdSkipped:self.networkModel adConfig:self.adConfig];
    }
    
    [PokktNetworkDelegate onAdClosed:self.networkModel adConfig:self.adConfig];
}


@end
